;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
PRO dispLab__Define
;********************************************************
;#structure#
; DispLabStruct
;#inheritance#
; NONE
;#description#
;internal variables for dispersedlab Fringe convenience object
;#structure_text#
fringe = {displab, $
INHERITS dispersedFringe, $
INHERITS spPhotoChop,  $
doPrint:0B, $
printFile:'', $
printUnit:0, $
dataFiles:strarr(3), $
fileListObj:OBJ_NEW(), $
objectName:'', $
labMaskFile:'', $
labFilterFile:'' $
}
;#end_structure#
;********************************************************
RETURN
END

PRO dispLab::save, file
   self->dispersedFringe::save, file
   self->spPhotoChop::save, file
   doPrint = self.doPrint
   printFile = self.printFile
   dataFiles = self.dataFiles
   objectName = self.objectName
   labMaskFile = self.labMaskFile
   labFilterFile = self.labFilterFile
   save,file=file+'_dispLab', doPrint, printFile, $
      dataFiles, objectName, labMaskFile, labFilterFile
RETURN
END

PRO dispLab::setPrint, doPrint
   if (self.printUnit NE 0) then begin
      FREE_LUN,self.printUnit
      self.printUnit = 0
   endif
   self.doPrint = doPrint
   self.printFile = '/tmp/EWS.'+STRTRIM(1000*randomu(seed,1),2)
   if (doPrint NE 0) then begin
      print,' printing into file ', self.printFile
      openw,self.printUnit,self.printFile,/GET_LUN
   endif
RETURN
END

PRO dispLab::output,s1
   print,s1
   if (self.doPrint GT 0) then printf,self.printUnit, s1
RETURN
END

PRO dispLab::print,printer=printer
   if (KEYWORD_SET(printer)) then pCom = ' lpr -P'+STRTRIM(printer,2)+' ' $
      else pCom = ' lpr '
   spawn,'a2ps '+self.printFile+' | pCom
RETURN
END
   
FUNCTION dispLab::init, file, gui=gui, config=config, doPrint=doPrint, $
   restore=restore
;********************************************************
;#function#
;dispLab
;#description#
;convenience object storing scanned fringe object and other information
; supposedly easier to deal with than raw scanned fringe object
;#call#
;FL = OBJECT_NEW('dispLab' ,maskfile)
;FL = OBJECT_NEW('dispLab', maskdir, /gui)
;FL = OBJECT_NEW('dispLab', configFile, /config)
;FL = OBJECT_NEW('dispLab', /config)
;#inputs#
;\anArg{maskfile}{str}{mask file name}
;\anArg{gui}{flag}{Use IDL gui to find mask file}
;\anArg{config}{flag}{use ASCII configuration file for mask/filter files}
;#return#
;\anArg{-}{dispLab object}{}
;#end_function#
;********************************************************
   if (KEYWORD_SET(restore)) then goto, RESTORE_ME
   self.printUnit = 0
   if (KEYWORD_SET(doPrint)) then self->setPrint,doPrint
   filterFile = ''
   if (KEYWORD_SET(config)) then begin
      if (N_PARAMS() EQ 1) then configFile = file $
;        else configFile = GETENV('ewsroot')+'/config.def'
        else configFile = GETENV('midiutil')+'/config.def'
      openr,unit,configFile,/GET_LUN
      maskfile = ''
      readf,unit, maskfile ; nonDispersed Mask
      readf,unit, maskfile
      self.labMaskFile = maskFile
      readf,unit, filterFile
      FREE_LUN, unit
   endif else if (NOT KEYWORD_SET(gui)) then $
      self.labMaskFile = maskfile $
   else begin
      if (N_PARAMS() LT 1) then maskDir = '.' else maskDir = maskFile
      f = DIALOG_PICKFILE(title='Pick Mask File', path=maskdir)
      self.labMaskFile = f
   endelse
   self->output,'MASK FILE IS   '+ self.labMaskFile
   self.labFilterFile = filterFile
   if (filterFile NE '') then self->output,'FILTER FILE IS '+filterFile $
      else self->output,'FILTER FILE NOT YET DEFINED'
   if (1 NE self->spPhotoChop::init(self.labMaskFile)) then RETURN,0
   if (1 NE self->dispersedFringe::init(self.labMaskFile)) then RETURN,0
RETURN,1
RESTORE_ME:
   if (1 NE self->dispersedFringe::init(file,/restore)) then RETURN,0
   if (1 NE self->spPhotoChop::init(file,/restore)) then RETURN,0
   RESTORE, file=file+'_dispLab'
   self.doPrint = doPrint
   self.printFile = printFile
   self.dataFiles = dataFiles
   self.objectName = objectName
   self.labMaskFile = labMaskFile
   self.labFilterFile = labFilterFile
RETURN,1
END

PRO dispLab::cleanup
   self->dispersedFringe::cleanup
   self->spPhotoChop::cleanup
   OBJ_DESTROY,self.fileListObj
RETURN
END

FUNCTION dispObj, file, gui=gui, config=config
;********************************************************
;#function#
;fringeObj
;#description#
;alias for OBJ_NEW(dispLab) for people who don't like OBJ_NEW syntax
;#call#
;z = fringeObj()
;z = fringeObj(maskdir)
;#inputs#
;\anArg{maskdir}{str}{directory where to start looking for mask files}
;\anArg{filterdir}{str}{directory where to start looking for filter files}
;#return#
;\anArg{-}{dispLab Object}{}
;#end_function#
;********************************************************
   if (N_PARAMS() EQ 1) then RETURN, $
      OBJ_NEW('dispLab', file , gui=gui, config=config)
   if (NOT KEYWORD_SET(gui)) AND (NOT KEYWORD_SET(config)) then config=1
   RETURN, OBJ_NEW('dispLab', gui=gui, config=config)
END

PRO dispLab::mask, maskfile,gui=gui
;********************************************************
;#procedure#
;mask
;#description#
;replace pixel mask file, optionally using gui
;#call#
;dispLabObj->mask,maskfile
;dispLabObj->mask,maskdir,/gui
;dispLabObj->mask,/gui (default to current dir)
;#inputs#
;\anArg{maskfile}{str}{name of maskfile}
;\anArg{maskdir}{str}{name of dir to start file GUI in}
;\anArg{gui}{flag}{use gui to find file}
;#end_procedure#
;********************************************************
   if (KEYWORD_SET(gui)) then begin
      if (N_PARAMS() LT 1) then maskdir = '.' else maskdir=maskfile
      self.maskfile = DIALOG_PICKFILE(title='Pick Mask File', path=maskdir)
   endif else begin
      if (N_PARAMS() LT 1) then begin
         self->output,'No mask file specified'
         RETURN
      endif
      self.maskfile = maskfile
   endelse
   PTR_FREE,self.maskData
   self->output,'Selected Mask File '+self.maskfile
   if (self.maskFile NE '') then self.maskData = $
      PTR_NEW(oirGetData(self.maskFile))
RETURN
END

PRO dispLab::getFilter, filterdir
;********************************************************
;#procedure#
;getFilter
;#description#
;use file GUI to select or replace spectral filter file
;#call#
;fringeObj->getFilter
;fringeObj->getFilter, filterdir
;z = fringeObj(maskdir)
;#inputs#
;\anArg{filterdir}{str}{directory where to start looking for filter files}
;#end_procedure#
;********************************************************
   if (N_PARAMS() LT 1) then filterdir = '.'
   f = DIALOG_PICKFILE(title='Pick filter File', path=filterdir)
   self.labfilterfile = f
   print,'FILTER FILE IS   ', self.filterfile
   self->scannedFringe::filter,f
RETURN
END

PRO dispLab::getFringeData, input, compressLast=compressLast, last=last,$
    gui=gui
;********************************************************
;#procedure#
;getFringeData
;#description#
;fetch fringe data for this object from disk, mask it and compress it
;and store internally.  If mask is not defined I will insist here
;#call#
;fringeObj->getFringeData, datafile
;fringeObj->getFringeData, dataDir,/gui
;fringeObj->getFringeData, /gui
;fringeObj->getData, /last
;fringeObj->getData, /compressLast
;#inputs#
;\anArg{datafile}{string}{datafile is /gui, /last and /comp not specified}
;\anArg{dataDir}{string}{directory to look for data; if /gui etc.default is "."}
;\anArg{gui}{flag}{use gui to find data}
;\anArg{last}{flag}{use most recent data file in directory}
;\anArg{last}{compressLast}{use most recent set of multiple files in directory}
;#end_procedure#
;********************************************************
; cleanup existing data
; decide where to get data
   IF (KEYWORD_SET(compressLast)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = gfu()
      fits = OBJ_NEW('fitsFileList', files)
      fits->midiCompress
      files = fits->files()
      print,'Compressing and storing ',files[0]
      dataFile = files[0]
      cd,currentdir
   ENDIF ELSE IF (KEYWORD_SET(last)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = gfu(last)
      print,'Compressing and storing ',last
      dataFile = last
      cd,currentdir
   ENDIF ELSE IF (KEYWORD_SET(gui)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = midigui()
      print,'Compressing and storing ',files[0]
      cd,currentdir
      dataFile = files[0]
   ENDIF ELSE dataFile = input
      print,'Getting Interferometric Data  '
      self->getFileData, dataFile
      self->getFitsData, dataFile
      self.dataFiles[2] = dataFile
      self->formFringes
RETURN
END

PRO dispLab::getPhotoData, file, gui=gui
   self.dataFiles[0:1] = file
   self->spPhotoChop::getData, file, gui=gui
RETURN
END

FUNCTION dispLab::maskData
RETURN,oirGetData(self.labMaskFile)
END

FUNCTION dispLab::waveNo
;********************************************************
;#function#
;waveNo	
;#description#
;returns array of waveNumbers (reciprocal wavelengths/microns)
; corresponding to power spectra abscissa
;#call#
;spFreq = labObject->outOpd()
;#inputs#
;#return#
;\anArg{-}{fltarr}{OPD array}
;#end_function#
;********************************************************
RETURN,*self.photoWaveNo
END

FUNCTION dispLab::fileObj
   RETURN, self.fileListObj
END

PRO dispLab::set,fileObj=fileObj, objName=objName
   if (KEYWORD_SET(fileObj)) then begin
      if (STRTRIM(OBJ_CLASS(fileObj),2) NE 'FITSFILELIST') then begin
         print,'fileobj isnt a fitsfilelist'
         RETURN
      endif
      self.fileListObj = fileObj
   endif
   if (KEYWORD_SET(objName)) then self.objectName = objName
RETURN
END

FUNCTION dispLab::objName
   RETURN,self.objectName
END

FUNCTION estimateVisDisp, input, files=files, oldGui=oldGui, $
   nosave=nosave, config=config, incoherent=incoherent, guidir=guidir
;
;  check whether input is an old dispLab object for recycling
   reCycle = KEYWORD_SET(oldGui)
   if (reCycle) then reCycle = (11 EQ SIZE(oldGui,/type))
   if (reCycle) then reCycle = (STRTRIM(OBJ_CLASS(oldGui),2) EQ 'DISPLAB') 
   if (reCycle) then begin
      fileObj = oldGui->fileObj()
      reCycle = (STRTRIM(OBJ_CLASS(fileObj),2) EQ 'FITSFILELIST')
   endif 
   if (N_PARAMS() GT 0) then z=obj_new('dispLab',input,/config)$
      else z=obj_new('dispLab',/config)
   if (N_ELEMENTS(files) GT 0) then d = files $
      else begin
         if (KEYWORD_SET(guidir)) then $
            dir= DIALOG_PICKFILE(title='Pick Directory') $
            else dir=''
         if (reCycle) then begin
            d=midiGui(fileObj,/restore) 
            z->set,fileObj=fileObj
         endif  else begin 
            d = midiGui(f,/save,dir=dir)
            z->set,fileObj=f
         endelse
      endelse
   if(N_ELEMENTS(d) NE 3) then begin
      print,'I WANT 3 and ONLY 3 files' 
      RETURN,0
   endif
   fileList = REPLICATE(-1,3)
   for i=0,2 do begin
      midiGetIns,d[i],shutter=shutter, grism=grism
      shutter = STRTRIM(shutter,2)
      if (shutter EQ 'AOPEN') then fileList[0]=i $
      else if (shutter EQ 'BOPEN') then fileList[1] = i $
      else if (shutter EQ 'ABOPEN') then fileList[2] = i
      if (STRTRIM(grism,2) NE 'PRISM') then begin
         print,'I only know how to deal with prism data'
         RETURN,0
      endif
   endfor
   if (TOTAL (fileList EQ (-1))) NE 0 then begin
      print,' I cant find two photo and one interf file'
      RETURN,0
   endif
   print,'PHOTO A file(s) ',d[fileList[0]]
   print,'PHOTO B file(s) ',d[fileList[1]]
   print,'FOURIER INTERF  file(s) ',d[fileList[2]]
   midigetINS,d[fileList[1]],object=object
   z->set,objName = object
   print,' OBJECT ',object
   z->getFringeData,d[fileList[2]]
   maskdata = z->maskData()
   z->getPhotodata,[d[fileList[0]], d[fileList[1]]]
   z->deSky
   print,'Rotate Fringes'
   z->rotate
   print,'Calculate Delays'
   z->delay
   print,'Auto select'
   z->autoSelect
   nFrames = N_ELEMENTS(z->time())
   nGood   = TOTAL(z->select())
   print,'Total Frames:',nFrames,' SelectedFrames:',nGood
   if(NOT KEYWORD_SET(incoherent)) then begin
      print,'Group Delay Rotation'
      z->groupRotate
      print,'Group Phase Zero Removal'
      z->dePhase
      weight=abs(z->coherentFringe())
      z->dePhase,weight=weight
   endif else begin 
      z->autoSpec
      z->autopower
   endelse
   if (KEYWORD_SET(nosave)) then begin
      OBJ_DESTROY,z
      z = OBJ_NEW()
   endif
RETURN,z
END

FUNCTION dispLab::files
   RETURN,self.dataFiles
END

FUNCTION dispLab::visibility
   if (NOT PTR_VALID(self.power)) OR (NOT PTR_VALID(self.maskFlux[2,0])) $
      OR (NOT PTR_VALID(self.maskFlux[2,1])) then begin
         print,' power or photometry not yet ready'
         RETURN,0
   endif
   RETURN,SQRT(*self.power*0.5>0)/((*self.maskFlux[2,0])+(*self.maskFlux[2,1])>.1)
END

FUNCTION dispLab::coVis
RETURN,self->coherentFringe()/(self->photoPower()>1)
END

FUNCTION dispLab::coPhase
   cf = self->coherentFringe()
RETURN,!radeg*midiPhase(cf)
END

PRO dispLab::report, file, insert=insert, calibrator=calibrator, doPrint=doPrint
   ll = ''
   if (NOT KEYWORD_SET(insert)) then begin
      self.doPrint = 0
      if (N_PARAMS() GT 0) then begin
         self.doPrint = 1
         set_plot,'ps'
         device, file=file, xs=25,ys=15,/landscape
      endif else set_plot,'x'
      oldX = !x.style
      oldY = !y.style
      !x.style = 1
      !y.style = 1
   endif else if (KEYWORD_SET(doPrint)) then self.doPrint=doPrint 
      
   if (KEYWORD_SET(calibrator)) then calibrator->report,/insert, doPrint=self.doPrint
   object = self->objName()
   lambda=self->lambda()
   self->photoData, mask=mask
   for i=0,1 do begin
      a = *mask[0,i]
      b = *mask[1,i]
     ab = *mask[2,i]
     mm = max(a) > max(b)
     PLOT, lambda, a, xr=[7.5, 13.5], xtitle='wavelength(mu)', ytitle='ADU/s/channel', title= $
        object+'  Photometry Channel '+STRTRIM(i+1,2), yr=[0, mm]
     OPLOT,lambda,b, line=2
     OPLOT,lambda,ab,thick=2
     if (self.doPrint EQ 0) then READ,ll
   endfor
   PLOT,lambda,self->photoPower(), xr=[7.5, 13.5], xtitle='wavelength(mu)', ytitle='ADU/s/channel', title= $
        object+'  Summed Photometry (I1+I2)'
   if (self.doPrint EQ 0) then READ,ll
   PLOT,lambda,ABS(self->coherentFringe()), xr=[7.5, 13.5], xtitle='wavelength(mu)', $
      ytitle='ADU/s/channel',  title = object+' Correlated Flux'
   if (self.doPrint EQ 0) then READ,ll
   PLOT,lambda,(self->coPhase()), xr=[7.5, 13.5], xtitle='wavelength(mu)', ytitle='degrees', $
      title= object+'  Instrumental Phase'
   if (self.doPrint EQ 0) then READ,ll
   PLOT,lambda,ABS(self->coVis()), xr=[7.5, 13.5], xtitle='wavelength(mu)', ytitle='Visibility', $
      title= object+'  Instrumental Visibility'
   if (self.doPrint EQ 0 AND (NOT KEYWORD_SET(calibrator))) then READ,ll
   if (KEYWORD_SET(calibrator)) then begin
      calvis = ABS(self->coVis())/(ABS(calibrator->coVis())>.01)
      PLOT,lambda, calvis, xr=[7.5, 13.5], $
      xtitle='wavelength(mu)', ytitle='Visibility', title= object+'  Calibrated Visibility', $
      yr = [0, max(calVis) < 1.5]
      if (self.doPrint EQ 0) then READ,ll
      calPhase = !radeg*midiPhase(self->coVis()/calibrator->coVis())
      PLOT,lambda, calPhase, xr=[7.5, 13.5], $
      xtitle='wavelength(mu)', ytitle='Phase', title= object+'  Calibrated Phase'
   endif
   if (NOT KEYWORD_SET(insert)) AND (self.doPrint EQ 1) then begin
      device,/close
      !x.style = oldX
      !y.style = oldY
      set_plot,'x'
   endif
RETURN
END
