;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: calibrate_disk.pro,v 1.6 2007/01/02 14:14:13 koehler Exp $
;
; calibrate_disk.pro
; Created:     Tue Jul 15 17:05:52 2003 by Koehler@sun47
; Last change: Tue Jan  2 14:55:49 2007
;
; PURPOSE:
;	calibrate the calibrators
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; cartesic method, i.e. Christoph's method
;
pro project_baseline, t, de, stations, base_len, base_pa
  ; t = hourangle in hours, de = Declination in degrees
  ; base_len is given in meters
  ; base_pa in degrees

  t  = t * !PI/12.
  de = de* !PI/180.
  sinH = sin(t)
  cosH = cos(t)

  ; from ESO exposure time calculator:
  ;longitude=  70.40479659;
  latitude = -24.62794830

  sin_la = sin(latitude * !PI/180.)
  cos_la = cos(latitude * !PI/180.)
  ;
  ; find out the baseline used
  ;
  case stations of
  "U1-U2": begin & BaselineLen=  56.569 & BaselinePA=  26.016 & end
  "U1-U3": begin & BaselineLen= 102.434 & BaselinePA=  32.369 & end
  "U1-U4": begin & BaselineLen= 130.231 & BaselinePA=  60.396 & end
  "U2-U3": begin & BaselineLen=  46.635 & BaselinePA=  40.082 & end
  "U2-U4": begin & BaselineLen=  89.443 & BaselinePA=  81.321 & end
  "U3-U4": begin & BaselineLen=  62.463 & BaselinePA= 110.803 & end
  "A0-B0": begin & BaselineLen=   8.010 & BaselinePA=   71.059 & end
  "A0-B1": begin & BaselineLen=  17.891 & BaselinePA=  134.419 & end
  "A0-B2": begin & BaselineLen=  25.299 & BaselinePA=  142.558 & end
  "A0-B3": begin & BaselineLen=  33.003 & BaselinePA=  146.970 & end
  "A0-B4": begin & BaselineLen=  40.794 & BaselinePA=  149.692 & end
  "A0-B5": begin & BaselineLen=  48.663 & BaselinePA=  151.542 & end
  "A0-C0": begin & BaselineLen=  15.999 & BaselinePA=   71.016 & end
  "A0-C1": begin & BaselineLen=  22.625 & BaselinePA=  116.014 & end
  "A0-C2": begin & BaselineLen=  28.849 & BaselinePA=  127.334 & end
  "A0-C3": begin & BaselineLen=  35.774 & BaselinePA=  134.450 & end
  "A0-D0": begin & BaselineLen=  32.011 & BaselinePA=   71.014 & end
  "A0-D1": begin & BaselineLen=  45.264 & BaselinePA=  116.008 & end
  "A0-D2": begin & BaselineLen=  57.694 & BaselinePA=  127.316 & end
  "A0-E0": begin & BaselineLen=  48.012 & BaselinePA=   71.020 & end
  "A0-G0": begin & BaselineLen=  64.018 & BaselinePA=   71.020 & end
  "A0-G1": begin & BaselineLen=  90.522 & BaselinePA=  116.005 & end
  "A0-G2": begin & BaselineLen=  68.352 & BaselinePA=   50.451 & end
  "A0-H0": begin & BaselineLen=  96.016 & BaselinePA=   71.012 & end
  "A0-I1": begin & BaselineLen= 111.423 & BaselinePA=   92.045 & end
  "A0-J1": begin & BaselineLen= 122.389 & BaselinePA=   82.315 & end
  "A0-J2": begin & BaselineLen= 129.257 & BaselinePA=   92.811 & end
  "A0-J3": begin & BaselineLen= 132.443 & BaselinePA=   45.999 & end
  "A0-J4": begin & BaselineLen= 139.961 & BaselinePA=   40.054 & end
  "A0-J5": begin & BaselineLen= 153.688 & BaselinePA=   32.360 & end
  "A0-J6": begin & BaselineLen= 169.720 & BaselinePA=   26.019 & end
  "A0-K0": begin & BaselineLen= 128.003 & BaselinePA=   71.013 & end
  "A0-L0": begin & BaselineLen= 136.022 & BaselinePA=   71.010 & end
  "A0-M0": begin & BaselineLen= 144.014 & BaselinePA=   71.011 & end
  "A0-U1": begin & BaselineLen=  35.789 & BaselinePA=    7.573 & end
  "A0-U2": begin & BaselineLen=  91.225 & BaselinePA=   18.886 & end
  "A0-U3": begin & BaselineLen= 135.756 & BaselinePA=   26.021 & end
  "A0-U4": begin & BaselineLen= 154.511 & BaselinePA=   49.761 & end
  "A1-B0": begin & BaselineLen=  17.895 & BaselinePA=    7.607 & end
  "A1-B1": begin & BaselineLen=   8.010 & BaselinePA=   70.944 & end
  "A1-B2": begin & BaselineLen=  11.314 & BaselinePA=  115.944 & end
  "A1-B3": begin & BaselineLen=  17.900 & BaselinePA=  134.434 & end
  "A1-B4": begin & BaselineLen=  25.294 & BaselinePA=  142.554 & end
  "A1-B5": begin & BaselineLen=  32.979 & BaselinePA=  146.959 & end
  "A1-C0": begin & BaselineLen=  22.632 & BaselinePA=   26.000 & end
  "A1-C1": begin & BaselineLen=  15.999 & BaselinePA=   70.980 & end
  "A1-C2": begin & BaselineLen=  17.887 & BaselinePA=   97.577 & end
  "A1-C3": begin & BaselineLen=  22.619 & BaselinePA=  115.998 & end
  "A1-D0": begin & BaselineLen=  35.791 & BaselinePA=   44.446 & end
  "A1-D1": begin & BaselineLen=  35.784 & BaselinePA=   97.565 & end
  "A1-D2": begin & BaselineLen=  45.256 & BaselinePA=  115.998 & end
  "A1-E0": begin & BaselineLen=  50.609 & BaselinePA=   52.580 & end
  "A1-G0": begin & BaselineLen=  65.988 & BaselinePA=   56.980 & end
  "A1-G1": begin & BaselineLen=  80.010 & BaselinePA=  107.871 & end
  "A1-G2": begin & BaselineLen=  75.478 & BaselinePA=   38.997 & end
  "A1-H0": begin & BaselineLen=  97.342 & BaselinePA=   61.547 & end
  "A1-I1": begin & BaselineLen= 106.730 & BaselinePA=   83.998 & end
  "A1-J1": begin & BaselineLen= 120.281 & BaselinePA=   74.816 & end
  "A1-J2": begin & BaselineLen= 124.206 & BaselinePA=   85.938 & end
  "A1-J3": begin & BaselineLen= 139.966 & BaselinePA=   40.050 & end
  "A1-J4": begin & BaselineLen= 148.831 & BaselinePA=   34.762 & end
  "A1-J5": begin & BaselineLen= 164.164 & BaselinePA=   27.993 & end
  "A1-J6": begin & BaselineLen= 181.392 & BaselinePA=   22.441 & end
  "A1-K0": begin & BaselineLen= 129.001 & BaselinePA=   63.885 & end
  "A1-L0": begin & BaselineLen= 136.962 & BaselinePA=   64.298 & end
  "A1-M0": begin & BaselineLen= 144.902 & BaselinePA=   64.668 & end
  "A1-U1": begin & BaselineLen=  50.617 & BaselinePA=   -0.555 & end
  "A1-U2": begin & BaselineLen= 104.325 & BaselinePA=   13.481 & end
  "A1-U3": begin & BaselineLen= 147.509 & BaselinePA=   21.620 & end
  "A1-U4": begin & BaselineLen= 161.007 & BaselinePA=   44.444 & end
  "B0-C0": begin & BaselineLen=   7.989 & BaselinePA=   70.973 & end
  "B0-C1": begin & BaselineLen=  17.876 & BaselinePA=  134.471 & end
  "B0-C2": begin & BaselineLen=  25.295 & BaselinePA=  142.605 & end
  "B0-C3": begin & BaselineLen=  32.973 & BaselinePA=  146.995 & end
  "B0-D0": begin & BaselineLen=  24.001 & BaselinePA=   70.999 & end
  "B0-D1": begin & BaselineLen=  39.997 & BaselinePA=  124.142 & end
  "B0-D2": begin & BaselineLen=  53.660 & BaselinePA=  134.447 & end
  "B0-E0": begin & BaselineLen=  40.002 & BaselinePA=   71.012 & end
  "B0-G0": begin & BaselineLen=  56.008 & BaselinePA=   71.014 & end
  "B0-G1": begin & BaselineLen=  85.042 & BaselinePA=  119.821 & end
  "B0-G2": begin & BaselineLen=  60.920 & BaselinePA=   47.798 & end
  "B0-H0": begin & BaselineLen=  88.006 & BaselinePA=   71.008 & end
  "B0-I1": begin & BaselineLen= 103.984 & BaselinePA=   93.626 & end
  "B0-J1": begin & BaselineLen= 114.544 & BaselinePA=   83.097 & end
  "B0-J2": begin & BaselineLen= 121.853 & BaselinePA=   94.207 & end
  "B0-J3": begin & BaselineLen= 125.233 & BaselinePA=   44.446 & end
  "B0-J4": begin & BaselineLen= 133.159 & BaselinePA=   38.278 & end
  "B0-J5": begin & BaselineLen= 147.522 & BaselinePA=   30.415 & end
  "B0-J6": begin & BaselineLen= 164.158 & BaselinePA=   24.041 & end
  "B0-K0": begin & BaselineLen= 119.993 & BaselinePA=   71.010 & end
  "B0-L0": begin & BaselineLen= 128.012 & BaselinePA=   71.007 & end
  "B0-M0": begin & BaselineLen= 136.004 & BaselinePA=   71.008 & end
  "B0-U1": begin & BaselineLen=  33.001 & BaselinePA=   -4.971 & end
  "B0-U2": begin & BaselineLen=  86.544 & BaselinePA=   14.694 & end
  "B0-U3": begin & BaselineLen= 130.219 & BaselinePA=   23.526 & end
  "B0-U4": begin & BaselineLen= 147.077 & BaselinePA=   48.628 & end
  "B1-C0": begin & BaselineLen=  17.882 & BaselinePA=    7.552 & end
  "B1-C1": begin & BaselineLen=   7.989 & BaselinePA=   71.016 & end
  "B1-C2": begin & BaselineLen=  11.312 & BaselinePA=  116.084 & end
  "B1-C3": begin & BaselineLen=  17.883 & BaselinePA=  134.481 & end
  "B1-D0": begin & BaselineLen=  28.845 & BaselinePA=   37.329 & end
  "B1-D1": begin & BaselineLen=  28.847 & BaselinePA=  104.712 & end
  "B1-D2": begin & BaselineLen=  40.001 & BaselinePA=  124.146 & end
  "B1-E0": begin & BaselineLen=  43.081 & BaselinePA=   49.222 & end
  "B1-G0": begin & BaselineLen=  58.247 & BaselinePA=   55.079 & end
  "B1-G1": begin & BaselineLen=  73.764 & BaselinePA=  111.611 & end
  "B1-G2": begin & BaselineLen=  68.812 & BaselinePA=   35.466 & end
  "B1-H0": begin & BaselineLen=  89.449 & BaselinePA=   60.709 & end
  "B1-I1": begin & BaselineLen=  98.943 & BaselinePA=   85.046 & end
  "B1-J1": begin & BaselineLen= 112.291 & BaselinePA=   75.092 & end
  "B1-J2": begin & BaselineLen= 116.487 & BaselinePA=   86.958 & end
  "B1-J3": begin & BaselineLen= 133.156 & BaselinePA=   38.280 & end
  "B1-J4": begin & BaselineLen= 142.444 & BaselinePA=   32.859 & end
  "B1-J5": begin & BaselineLen= 158.395 & BaselinePA=   26.018 & end
  "B1-J6": begin & BaselineLen= 176.187 & BaselinePA=   20.490 & end
  "B1-K0": begin & BaselineLen= 121.056 & BaselinePA=   63.419 & end
  "B1-L0": begin & BaselineLen= 129.010 & BaselinePA=   63.886 & end
  "B1-M0": begin & BaselineLen= 136.943 & BaselinePA=   64.302 & end
  "B1-U1": begin & BaselineLen=  48.671 & BaselinePA=   -9.534 & end
  "B1-U2": begin & BaselineLen= 100.245 & BaselinePA=    9.619 & end
  "B1-U3": begin & BaselineLen= 142.418 & BaselinePA=   19.175 & end
  "B1-U4": begin & BaselineLen= 153.880 & BaselinePA=   43.114 & end
  "B2-C0": begin & BaselineLen=  25.293 & BaselinePA=   -0.571 & end
  "B2-C1": begin & BaselineLen=  11.306 & BaselinePA=   25.977 & end
  "B2-C2": begin & BaselineLen=   7.989 & BaselinePA=   71.073 & end
  "B2-C3": begin & BaselineLen=  11.305 & BaselinePA=  116.052 & end
  "B2-D0": begin & BaselineLen=  33.941 & BaselinePA=   26.018 & end
  "B2-D1": begin & BaselineLen=  25.300 & BaselinePA=   89.459 & end
  "B2-D2": begin & BaselineLen=  33.943 & BaselinePA=  116.016 & end
  "B2-E0": begin & BaselineLen=  46.647 & BaselinePA=   40.059 & end
  "B2-G0": begin & BaselineLen=  60.931 & BaselinePA=   47.826 & end
  "B2-G1": begin & BaselineLen=  68.827 & BaselinePA=  106.548 & end
  "B2-G2": begin & BaselineLen=  73.751 & BaselinePA=   30.403 & end
  "B2-H0": begin & BaselineLen=  91.221 & BaselinePA=   55.759 & end
  "B2-I1": begin & BaselineLen=  97.314 & BaselinePA=   80.471 & end
  "B2-J1": begin & BaselineLen= 112.007 & BaselinePA=   71.006 & end
  "B2-J2": begin & BaselineLen= 114.548 & BaselinePA=   83.107 & end
  "B2-J3": begin & BaselineLen= 137.647 & BaselinePA=   35.478 & end
  "B2-J4": begin & BaselineLen= 147.521 & BaselinePA=   30.415 & end
  "B2-J5": begin & BaselineLen= 164.150 & BaselinePA=   24.043 & end
  "B2-J6": begin & BaselineLen= 182.433 & BaselinePA=   18.893 & end
  "B2-K0": begin & BaselineLen= 122.371 & BaselinePA=   59.703 & end
  "B2-L0": begin & BaselineLen= 130.245 & BaselinePA=   60.392 & end
  "B2-M0": begin & BaselineLen= 138.107 & BaselinePA=   61.004 & end
  "B2-U1": begin & BaselineLen=  56.578 & BaselinePA=  -10.864 & end
  "B2-U2": begin & BaselineLen= 107.337 & BaselinePA=    7.574 & end
  "B2-U3": begin & BaselineLen= 148.791 & BaselinePA=   17.271 & end
  "B2-U4": begin & BaselineLen= 157.782 & BaselinePA=   40.545 & end
  "B3-C0": begin & BaselineLen=  32.998 & BaselinePA=   -4.973 & end
  "B3-C1": begin & BaselineLen=  17.900 & BaselinePA=    7.524 & end
  "B3-C2": begin & BaselineLen=  11.313 & BaselinePA=   25.941 & end
  "B3-C3": begin & BaselineLen=   7.989 & BaselinePA=   70.880 & end
  "B3-D0": begin & BaselineLen=  40.014 & BaselinePA=   17.872 & end
  "B3-D1": begin & BaselineLen=  24.001 & BaselinePA=   70.983 & end
  "B3-D2": begin & BaselineLen=  28.836 & BaselinePA=  104.677 & end
  "B3-E0": begin & BaselineLen=  51.235 & BaselinePA=   32.346 & end
  "B3-G0": begin & BaselineLen=  64.511 & BaselinePA=   41.266 & end
  "B3-G1": begin & BaselineLen=  64.498 & BaselinePA=  100.741 & end
  "B3-G2": begin & BaselineLen=  79.204 & BaselinePA=   25.996 & end
  "B3-H0": begin & BaselineLen=  93.651 & BaselinePA=   51.021 & end
  "B3-I1": begin & BaselineLen=  96.322 & BaselinePA=   75.761 & end
  "B3-J1": begin & BaselineLen= 112.295 & BaselinePA=   66.912 & end
  "B3-J2": begin & BaselineLen= 113.141 & BaselinePA=   79.134 & end
  "B3-J3": begin & BaselineLen= 142.457 & BaselinePA=   32.853 & end
  "B3-J4": begin & BaselineLen= 152.860 & BaselinePA=   28.133 & end
  "B3-J5": begin & BaselineLen= 170.099 & BaselinePA=   22.200 & end
  "B3-J6": begin & BaselineLen= 188.826 & BaselinePA=   17.399 & end
  "B3-K0": begin & BaselineLen= 124.193 & BaselinePA=   56.073 & end
  "B3-L0": begin & BaselineLen= 131.959 & BaselinePA=   56.968 & end
  "B3-M0": begin & BaselineLen= 139.725 & BaselinePA=   57.764 & end
  "B3-U1": begin & BaselineLen=  64.526 & BaselinePA=  -11.870 & end
  "B3-U2": begin & BaselineLen= 114.565 & BaselinePA=    5.781 & end
  "B3-U3": begin & BaselineLen= 155.329 & BaselinePA=   15.522 & end
  "B3-U4": begin & BaselineLen= 161.996 & BaselinePA=   38.100 & end
  "B4-C0": begin & BaselineLen=  40.790 & BaselinePA=   -7.689 & end
  "B4-C1": begin & BaselineLen=  25.297 & BaselinePA=   -0.574 & end
  "B4-C2": begin & BaselineLen=  17.878 & BaselinePA=    7.558 & end
  "B4-C3": begin & BaselineLen=  11.308 & BaselinePA=   25.966 & end
  "B4-D0": begin & BaselineLen=  46.649 & BaselinePA=   11.980 & end
  "B4-D1": begin & BaselineLen=  25.299 & BaselinePA=   52.586 & end
  "B4-D2": begin & BaselineLen=  25.299 & BaselinePA=   89.448 & end
  "B4-E0": begin & BaselineLen=  56.568 & BaselinePA=   26.020 & end
  "B4-G0": begin & BaselineLen=  68.823 & BaselinePA=   35.485 & end
  "B4-G1": begin & BaselineLen=  60.935 & BaselinePA=   94.208 & end
  "B4-G2": begin & BaselineLen=  85.039 & BaselinePA=   22.190 & end
  "B4-H0": begin & BaselineLen=  96.672 & BaselinePA=   46.570 & end
  "B4-I1": begin & BaselineLen=  95.992 & BaselinePA=   71.006 & end
  "B4-J1": begin & BaselineLen= 113.147 & BaselinePA=   62.876 & end
  "B4-J2": begin & BaselineLen= 112.292 & BaselinePA=   75.097 & end
  "B4-J3": begin & BaselineLen= 147.524 & BaselinePA=   30.414 & end
  "B4-J4": begin & BaselineLen= 158.401 & BaselinePA=   26.016 & end
  "B4-J5": begin & BaselineLen= 176.186 & BaselinePA=   20.490 & end
  "B4-J6": begin & BaselineLen= 195.311 & BaselinePA=   16.009 & end
  "B4-K0": begin & BaselineLen= 126.487 & BaselinePA=   52.577 & end
  "B4-L0": begin & BaselineLen= 134.120 & BaselinePA=   53.657 & end
  "B4-M0": begin & BaselineLen= 141.768 & BaselinePA=   54.622 & end
  "B4-U1": begin & BaselineLen=  72.455 & BaselinePA=  -12.652 & end
  "B4-U2": begin & BaselineLen= 121.861 & BaselinePA=    4.208 & end
  "B4-U3": begin & BaselineLen= 161.971 & BaselinePA=   13.922 & end
  "B4-U4": begin & BaselineLen= 166.469 & BaselinePA=   35.793 & end
  "B5-C0": begin & BaselineLen=  48.659 & BaselinePA=   -9.534 & end
  "B5-C1": begin & BaselineLen=  32.983 & BaselinePA=   -4.967 & end
  "B5-C2": begin & BaselineLen=  25.288 & BaselinePA=   -0.568 & end
  "B5-C3": begin & BaselineLen=  17.885 & BaselinePA=    7.547 & end
  "B5-D0": begin & BaselineLen=  53.666 & BaselinePA=    7.582 & end
  "B5-D1": begin & BaselineLen=  28.844 & BaselinePA=   37.332 & end
  "B5-D2": begin & BaselineLen=  24.001 & BaselinePA=   71.016 & end
  "B5-E0": begin & BaselineLen=  62.480 & BaselinePA=   20.825 & end
  "B5-G0": begin & BaselineLen=  73.759 & BaselinePA=   30.422 & end
  "B5-G1": begin & BaselineLen=  58.251 & BaselinePA=   86.957 & end
  "B5-G2": begin & BaselineLen=  91.213 & BaselinePA=   18.881 & end
  "B5-H0": begin & BaselineLen= 100.247 & BaselinePA=   42.405 & end
  "B5-I1": begin & BaselineLen=  96.326 & BaselinePA=   66.243 & end
  "B5-J1": begin & BaselineLen= 114.554 & BaselinePA=   58.912 & end
  "B5-J2": begin & BaselineLen= 112.007 & BaselinePA=   71.012 & end
  "B5-J3": begin & BaselineLen= 152.850 & BaselinePA=   28.137 & end
  "B5-J4": begin & BaselineLen= 164.155 & BaselinePA=   24.042 & end
  "B5-J5": begin & BaselineLen= 182.432 & BaselinePA=   18.893 & end
  "B5-J6": begin & BaselineLen= 201.916 & BaselinePA=   14.707 & end
  "B5-K0": begin & BaselineLen= 129.240 & BaselinePA=   49.211 & end
  "B5-L0": begin & BaselineLen= 136.720 & BaselinePA=   50.456 & end
  "B5-M0": begin & BaselineLen= 144.230 & BaselinePA=   51.572 & end
  "B5-U1": begin & BaselineLen=  80.410 & BaselinePA=  -13.281 & end
  "B5-U2": begin & BaselineLen= 129.252 & BaselinePA=    2.812 & end
  "B5-U3": begin & BaselineLen= 168.743 & BaselinePA=   12.446 & end
  "B5-U4": begin & BaselineLen= 171.208 & BaselinePA=   33.606 & end
  "C0-D0": begin & BaselineLen=  16.012 & BaselinePA=   71.012 & end
  "C0-D1": begin & BaselineLen=  35.784 & BaselinePA=  134.435 & end
  "C0-D2": begin & BaselineLen=  50.599 & BaselinePA=  142.568 & end
  "C0-E0": begin & BaselineLen=  32.013 & BaselinePA=   71.021 & end
  "C0-G0": begin & BaselineLen=  48.019 & BaselinePA=   71.021 & end
  "C0-G1": begin & BaselineLen=  80.011 & BaselinePA=  124.132 & end
  "C0-G2": begin & BaselineLen=  53.667 & BaselinePA=   44.440 & end
  "C0-H0": begin & BaselineLen=  80.017 & BaselinePA=   71.012 & end
  "C0-I1": begin & BaselineLen=  96.660 & BaselinePA=   95.451 & end
  "C0-J1": begin & BaselineLen= 106.746 & BaselinePA=   83.997 & end
  "C0-J2": begin & BaselineLen= 114.556 & BaselinePA=   95.784 & end
  "C0-J3": begin & BaselineLen= 118.139 & BaselinePA=   42.716 & end
  "C0-J4": begin & BaselineLen= 126.509 & BaselinePA=   36.323 & end
  "C0-J5": begin & BaselineLen= 141.548 & BaselinePA=   28.312 & end
  "C0-J6": begin & BaselineLen= 158.810 & BaselinePA=   21.935 & end
  "C0-K0": begin & BaselineLen= 112.004 & BaselinePA=   71.012 & end
  "C0-L0": begin & BaselineLen= 120.023 & BaselinePA=   71.009 & end
  "C0-M0": begin & BaselineLen= 128.015 & BaselinePA=   71.010 & end
  "C0-U1": begin & BaselineLen=  32.013 & BaselinePA=  -18.980 & end
  "C0-U2": begin & BaselineLen=  82.377 & BaselinePA=   10.067 & end
  "C0-U3": begin & BaselineLen= 124.955 & BaselinePA=   20.827 & end
  "C0-U4": begin & BaselineLen= 139.721 & BaselinePA=   47.382 & end
  "C1-D0": begin & BaselineLen=  22.635 & BaselinePA=   26.039 & end
  "C1-D1": begin & BaselineLen=  22.639 & BaselinePA=  116.002 & end
  "C1-D2": begin & BaselineLen=  35.783 & BaselinePA=  134.434 & end
  "C1-E0": begin & BaselineLen=  35.786 & BaselinePA=   44.467 & end
  "C1-G0": begin & BaselineLen=  50.613 & BaselinePA=   52.595 & end
  "C1-G1": begin & BaselineLen=  67.897 & BaselinePA=  116.002 & end
  "C1-G2": begin & BaselineLen=  62.485 & BaselinePA=   31.203 & end
  "C1-H0": begin & BaselineLen=  81.602 & BaselinePA=   59.706 & end
  "C1-I1": begin & BaselineLen=  91.213 & BaselinePA=   86.262 & end
  "C1-J1": begin & BaselineLen= 104.324 & BaselinePA=   75.404 & end
  "C1-J2": begin & BaselineLen= 108.827 & BaselinePA=   88.113 & end
  "C1-J3": begin & BaselineLen= 126.510 & BaselinePA=   36.323 & end
  "C1-J4": begin & BaselineLen= 136.251 & BaselinePA=   30.783 & end
  "C1-J5": begin & BaselineLen= 152.851 & BaselinePA=   23.900 & end
  "C1-J6": begin & BaselineLen= 171.219 & BaselinePA=   18.426 & end
  "C1-K0": begin & BaselineLen= 113.142 & BaselinePA=   62.884 & end
  "C1-L0": begin & BaselineLen= 121.086 & BaselinePA=   63.417 & end
  "C1-M0": begin & BaselineLen= 129.012 & BaselinePA=   63.887 & end
  "C1-U1": begin & BaselineLen=  48.011 & BaselinePA=  -18.982 & end
  "C1-U2": begin & BaselineLen=  96.675 & BaselinePA=    5.458 & end
  "C1-U3": begin & BaselineLen= 137.626 & BaselinePA=   16.559 & end
  "C1-U4": begin & BaselineLen= 146.868 & BaselinePA=   41.655 & end
  "C2-D0": begin & BaselineLen=  28.857 & BaselinePA=   14.718 & end
  "C2-D1": begin & BaselineLen=  17.897 & BaselinePA=   97.552 & end
  "C2-D2": begin & BaselineLen=  28.845 & BaselinePA=  127.298 & end
  "C2-E0": begin & BaselineLen=  40.012 & BaselinePA=   34.154 & end
  "C2-G0": begin & BaselineLen=  53.683 & BaselinePA=   44.458 & end
  "C2-G1": begin & BaselineLen=  62.493 & BaselinePA=  110.802 & end
  "C2-G2": begin & BaselineLen=  67.891 & BaselinePA=   26.005 & end
  "C2-H0": begin & BaselineLen=  83.542 & BaselinePA=   54.312 & end
  "C2-I1": begin & BaselineLen=  89.442 & BaselinePA=   81.307 & end
  "C2-J1": begin & BaselineLen= 104.018 & BaselinePA=   71.001 & end
  "C2-J2": begin & BaselineLen= 106.748 & BaselinePA=   84.001 & end
  "C2-J3": begin & BaselineLen= 131.233 & BaselinePA=   33.447 & end
  "C2-J4": begin & BaselineLen= 141.556 & BaselinePA=   28.308 & end
  "C2-J5": begin & BaselineLen= 158.812 & BaselinePA=   21.934 & end
  "C2-J6": begin & BaselineLen= 177.647 & BaselinePA=   16.857 & end
  "C2-K0": begin & BaselineLen= 114.549 & BaselinePA=   58.915 & end
  "C2-L0": begin & BaselineLen= 122.403 & BaselinePA=   59.699 & end
  "C2-M0": begin & BaselineLen= 130.249 & BaselinePA=   60.389 & end
  "C2-U1": begin & BaselineLen=  56.019 & BaselinePA=  -18.982 & end
  "C2-U2": begin & BaselineLen= 104.018 & BaselinePA=    3.633 & end
  "C2-U3": begin & BaselineLen= 144.217 & BaselinePA=   14.709 & end
  "C2-U4": begin & BaselineLen= 150.955 & BaselinePA=   39.005 & end
  "C3-D0": begin & BaselineLen=  35.781 & BaselinePA=    7.600 & end
  "C3-D1": begin & BaselineLen=  16.012 & BaselinePA=   71.034 & end
  "C3-D2": begin & BaselineLen=  22.637 & BaselinePA=  115.998 & end
  "C3-E0": begin & BaselineLen=  45.260 & BaselinePA=   26.033 & end
  "C3-G0": begin & BaselineLen=  57.701 & BaselinePA=   37.342 & end
  "C3-G1": begin & BaselineLen=  57.707 & BaselinePA=  104.694 & end
  "C3-G2": begin & BaselineLen=  73.760 & BaselinePA=   21.612 & end
  "C3-H0": begin & BaselineLen=  86.180 & BaselinePA=   49.217 & end
  "C3-I1": begin & BaselineLen=  88.365 & BaselinePA=   76.202 & end
  "C3-J1": begin & BaselineLen= 104.327 & BaselinePA=   66.608 & end
  "C3-J2": begin & BaselineLen= 105.241 & BaselinePA=   79.758 & end
  "C3-J3": begin & BaselineLen= 136.253 & BaselinePA=   30.783 & end
  "C3-J4": begin & BaselineLen= 147.093 & BaselinePA=   26.020 & end
  "C3-J5": begin & BaselineLen= 164.933 & BaselinePA=   20.115 & end
  "C3-J6": begin & BaselineLen= 184.184 & BaselinePA=   15.401 & end
  "C3-K0": begin & BaselineLen= 116.487 & BaselinePA=   55.069 & end
  "C3-L0": begin & BaselineLen= 124.219 & BaselinePA=   56.082 & end
  "C3-M0": begin & BaselineLen= 131.956 & BaselinePA=   56.977 & end
  "C3-U1": begin & BaselineLen=  64.010 & BaselinePA=  -18.982 & end
  "C3-U2": begin & BaselineLen= 111.437 & BaselinePA=    2.053 & end
  "C3-U3": begin & BaselineLen= 150.931 & BaselinePA=   13.026 & end
  "C3-U4": begin & BaselineLen= 155.339 & BaselinePA=   36.505 & end
  "D0-E0": begin & BaselineLen=  16.001 & BaselinePA=   71.030 & end
  "D0-G0": begin & BaselineLen=  32.007 & BaselinePA=   71.025 & end
  "D0-G1": begin & BaselineLen=  71.557 & BaselinePA=  134.443 & end
  "D0-G2": begin & BaselineLen=  39.993 & BaselinePA=   34.123 & end
  "D0-H0": begin & BaselineLen=  64.005 & BaselinePA=   71.012 & end
  "D0-I1": begin & BaselineLen=  82.350 & BaselinePA=  100.065 & end
  "D0-J1": begin & BaselineLen=  91.215 & BaselinePA=   86.258 & end
  "D0-J2": begin & BaselineLen= 100.242 & BaselinePA=   99.621 & end
  "D0-J3": begin & BaselineLen= 104.317 & BaselinePA=   38.543 & end
  "D0-J4": begin & BaselineLen= 113.709 & BaselinePA=   31.727 & end
  "D0-J5": begin & BaselineLen= 130.234 & BaselinePA=   23.529 & end
  "D0-J6": begin & BaselineLen= 148.814 & BaselinePA=   17.271 & end
  "D0-K0": begin & BaselineLen=  95.992 & BaselinePA=   71.012 & end
  "D0-L0": begin & BaselineLen= 104.011 & BaselinePA=   71.008 & end
  "D0-M0": begin & BaselineLen= 112.003 & BaselinePA=   71.010 & end
  "D0-U1": begin & BaselineLen=  35.792 & BaselinePA=  -45.555 & end
  "D0-U2": begin & BaselineLen=  75.903 & BaselinePA=   -0.559 & end
  "D0-U3": begin & BaselineLen= 115.360 & BaselinePA=   14.707 & end
  "D0-U4": begin & BaselineLen= 125.216 & BaselinePA=   44.444 & end
  "D1-E0": begin & BaselineLen=  35.777 & BaselinePA=    7.583 & end
  "D1-G0": begin & BaselineLen=  45.258 & BaselinePA=   26.024 & end
  "D1-G1": begin & BaselineLen=  45.258 & BaselinePA=  116.003 & end
  "D1-G2": begin & BaselineLen=  64.501 & BaselinePA=   10.744 & end
  "D1-H0": begin & BaselineLen=  71.562 & BaselinePA=   44.447 & end
  "D1-I1": begin & BaselineLen=  72.432 & BaselinePA=   77.343 & end
  "D1-J1": begin & BaselineLen=  88.371 & BaselinePA=   65.807 & end
  "D1-J2": begin & BaselineLen=  89.447 & BaselinePA=   81.314 & end
  "D1-J3": begin & BaselineLen= 124.463 & BaselinePA=   26.014 & end
  "D1-J4": begin & BaselineLen= 136.245 & BaselinePA=   21.252 & end
  "D1-J5": begin & BaselineLen= 155.337 & BaselinePA=   15.526 & end
  "D1-J6": begin & BaselineLen= 175.643 & BaselinePA=   11.085 & end
  "D1-K0": begin & BaselineLen= 101.188 & BaselinePA=   52.575 & end
  "D1-L0": begin & BaselineLen= 108.827 & BaselinePA=   53.906 & end
  "D1-M0": begin & BaselineLen= 116.489 & BaselinePA=   55.064 & end
  "D1-U1": begin & BaselineLen=  65.987 & BaselinePA=  -33.026 & end
  "D1-U2": begin & BaselineLen= 106.746 & BaselinePA=   -5.996 & end
  "D1-U3": begin & BaselineLen= 143.093 & BaselinePA=    7.580 & end
  "D1-U4": begin & BaselineLen= 142.437 & BaselinePA=   32.852 & end
  "D2-E0": begin & BaselineLen=  50.593 & BaselinePA=   -0.547 & end
  "D2-G0": begin & BaselineLen=  57.688 & BaselinePA=   14.715 & end
  "D2-G1": begin & BaselineLen=  35.785 & BaselinePA=   97.571 & end
  "D2-G2": begin & BaselineLen=  78.793 & BaselinePA=    4.966 & end
  "D2-H0": begin & BaselineLen=  80.007 & BaselinePA=   34.145 & end
  "D2-I1": begin & BaselineLen=  72.436 & BaselinePA=   64.663 & end
  "D2-J1": begin & BaselineLen=  91.225 & BaselinePA=   55.750 & end
  "D2-J2": begin & BaselineLen=  88.006 & BaselinePA=   71.011 & end
  "D2-J3": begin & BaselineLen= 136.245 & BaselinePA=   21.252 & end
  "D2-J4": begin & BaselineLen= 148.816 & BaselinePA=   17.271 & end
  "D2-J5": begin & BaselineLen= 168.762 & BaselinePA=   12.447 & end
  "D2-J6": begin & BaselineLen= 189.657 & BaselinePA=    8.663 & end
  "D2-K0": begin & BaselineLen= 107.327 & BaselinePA=   44.446 & end
  "D2-L0": begin & BaselineLen= 114.558 & BaselinePA=   46.237 & end
  "D2-M0": begin & BaselineLen= 121.860 & BaselinePA=   47.813 & end
  "D2-U1": begin & BaselineLen=  81.598 & BaselinePA=  -30.299 & end
  "D2-U2": begin & BaselineLen= 122.386 & BaselinePA=   -7.680 & end
  "D2-U3": begin & BaselineLen= 157.564 & BaselinePA=    4.978 & end
  "D2-U4": begin & BaselineLen= 152.841 & BaselinePA=   28.131 & end
  "E0-G0": begin & BaselineLen=  16.006 & BaselinePA=   71.020 & end
  "E0-G1": begin & BaselineLen=  65.966 & BaselinePA=  146.971 & end
  "E0-G2": begin & BaselineLen=  28.846 & BaselinePA=   14.665 & end
  "E0-H0": begin & BaselineLen=  48.004 & BaselinePA=   71.005 & end
  "E0-I1": begin & BaselineLen=  68.799 & BaselinePA=  106.546 & end
  "E0-J1": begin & BaselineLen=  75.892 & BaselinePA=   89.433 & end
  "E0-J2": begin & BaselineLen=  86.531 & BaselinePA=  104.698 & end
  "E0-J3": begin & BaselineLen=  91.225 & BaselinePA=   33.137 & end
  "E0-J4": begin & BaselineLen= 101.833 & BaselinePA=   26.014 & end
  "E0-J5": begin & BaselineLen= 120.005 & BaselinePA=   17.887 & end
  "E0-J6": begin & BaselineLen= 139.951 & BaselinePA=   11.980 & end
  "E0-K0": begin & BaselineLen=  79.991 & BaselinePA=   71.009 & end
  "E0-L0": begin & BaselineLen=  88.010 & BaselinePA=   71.004 & end
  "E0-M0": begin & BaselineLen=  96.002 & BaselinePA=   71.006 & end
  "E0-U1": begin & BaselineLen=  45.274 & BaselinePA=  -63.980 & end
  "E0-U2": begin & BaselineLen=  72.458 & BaselinePA=  -12.654 & end
  "E0-U3": begin & BaselineLen= 107.317 & BaselinePA=    7.579 & end
  "E0-U4": begin & BaselineLen= 111.138 & BaselinePA=   40.750 & end
  "G0-H0": begin & BaselineLen=  31.998 & BaselinePA=   70.998 & end
  "G0-I1": begin & BaselineLen=  56.543 & BaselinePA=  116.013 & end
  "G0-J1": begin & BaselineLen=  60.915 & BaselinePA=   94.193 & end
  "G0-J2": begin & BaselineLen=  73.748 & BaselinePA=  111.611 & end
  "G0-J3": begin & BaselineLen=  79.205 & BaselinePA=   26.009 & end
  "G0-J4": begin & BaselineLen=  91.221 & BaselinePA=   18.887 & end
  "G0-J5": begin & BaselineLen= 111.143 & BaselinePA=   11.271 & end
  "G0-J6": begin & BaselineLen= 132.430 & BaselinePA=    6.031 & end
  "G0-K0": begin & BaselineLen=  63.985 & BaselinePA=   71.006 & end
  "G0-L0": begin & BaselineLen=  72.004 & BaselinePA=   71.001 & end
  "G0-M0": begin & BaselineLen=  79.996 & BaselinePA=   71.004 & end
  "G0-U1": begin & BaselineLen=  57.713 & BaselinePA=  -75.289 & end
  "G0-U2": begin & BaselineLen=  72.462 & BaselinePA=  -25.336 & end
  "G0-U3": begin & BaselineLen= 101.178 & BaselinePA=   -0.556 & end
  "G0-U4": begin & BaselineLen=  97.648 & BaselinePA=   36.010 & end
  "G1-H0": begin & BaselineLen=  71.555 & BaselinePA=    7.576 & end
  "G1-I1": begin & BaselineLen=  46.638 & BaselinePA=   40.027 & end
  "G1-J1": begin & BaselineLen=  68.826 & BaselinePA=   35.464 & end
  "G1-J2": begin & BaselineLen=  58.238 & BaselinePA=   55.065 & end
  "G1-J3": begin & BaselineLen= 132.427 & BaselinePA=    6.030 & end
  "G1-J4": begin & BaselineLen= 147.079 & BaselinePA=    3.394 & end
  "G1-J5": begin & BaselineLen= 169.513 & BaselinePA=    0.305 & end
  "G1-J6": begin & BaselineLen= 192.332 & BaselinePA=   -2.058 & end
  "G1-K0": begin & BaselineLen=  90.500 & BaselinePA=   26.006 & end
  "G1-L0": begin & BaselineLen=  96.341 & BaselinePA=   29.378 & end
  "G1-M0": begin & BaselineLen= 102.451 & BaselinePA=   32.349 & end
  "G1-U1": begin & BaselineLen= 107.349 & BaselinePA=  -45.556 & end
  "G1-U2": begin & BaselineLen= 136.246 & BaselinePA=  -22.359 & end
  "G1-U3": begin & BaselineLen= 163.148 & BaselinePA=   -7.679 & end
  "G1-U4": begin & BaselineLen= 144.219 & BaselinePA=   14.697 & end
  "G2-H0": begin & BaselineLen=  40.018 & BaselinePA=  107.873 & end
  "G2-I1": begin & BaselineLen=  75.470 & BaselinePA=  129.004 & end
  "G2-J1": begin & BaselineLen=  73.765 & BaselinePA=  111.600 & end
  "G2-J2": begin & BaselineLen=  91.229 & BaselinePA=  123.131 & end
  "G2-J3": begin & BaselineLen=  64.516 & BaselinePA=   41.281 & end
  "G2-J4": begin & BaselineLen=  73.770 & BaselinePA=   30.428 & end
  "G2-J5": begin & BaselineLen=  91.219 & BaselinePA=   18.905 & end
  "G2-J6": begin & BaselineLen= 111.144 & BaselinePA=   11.284 & end
  "G2-K0": begin & BaselineLen=  68.360 & BaselinePA=   91.572 & end
  "G2-L0": begin & BaselineLen=  75.918 & BaselinePA=   89.441 & end
  "G2-M0": begin & BaselineLen=  83.539 & BaselinePA=   87.710 & end
  "G2-U1": begin & BaselineLen=  48.658 & BaselinePA=  -99.517 & end
  "G2-U2": begin & BaselineLen=  48.664 & BaselinePA=  -28.440 & end
  "G2-U3": begin & BaselineLen=  78.771 & BaselinePA=    4.990 & end
  "G2-U4": begin & BaselineLen=  86.168 & BaselinePA=   49.214 & end
  "H0-I1": begin & BaselineLen=  40.780 & BaselinePA=  149.723 & end
  "H0-J1": begin & BaselineLen=  33.931 & BaselinePA=  115.997 & end
  "H0-J2": begin & BaselineLen=  53.664 & BaselinePA=  134.449 & end
  "H0-J3": begin & BaselineLen=  60.929 & BaselinePA=    4.214 & end
  "H0-J4": begin & BaselineLen=  75.895 & BaselinePA=   -0.548 & end
  "H0-J5": begin & BaselineLen=  98.949 & BaselinePA=   -4.946 & end
  "H0-J6": begin & BaselineLen= 122.374 & BaselinePA=   -7.673 & end
  "H0-K0": begin & BaselineLen=  31.987 & BaselinePA=   71.014 & end
  "H0-L0": begin & BaselineLen=  40.006 & BaselinePA=   71.003 & end
  "H0-M0": begin & BaselineLen=  47.998 & BaselinePA=   71.008 & end
  "H0-U1": begin & BaselineLen=  86.179 & BaselinePA=  -87.182 & end
  "H0-U2": begin & BaselineLen=  82.378 & BaselinePA=  -48.045 & end
  "H0-U3": begin & BaselineLen=  95.980 & BaselinePA=  -18.992 & end
  "H0-U4": begin & BaselineLen=  73.752 & BaselinePA=   21.605 & end
  "I1-J1": begin & BaselineLen=  22.642 & BaselinePA=   26.034 & end
  "I1-J2": begin & BaselineLen=  17.906 & BaselinePA=   97.582 & end
  "I1-J3": begin & BaselineLen=  97.320 & BaselinePA=   -9.512 & end
  "I1-J4": begin & BaselineLen= 113.129 & BaselinePA=  -10.846 & end
  "I1-J5": begin & BaselineLen= 136.924 & BaselinePA=  -12.267 & end
  "I1-J6": begin & BaselineLen= 160.787 & BaselinePA=  -13.268 & end
  "I1-K0": begin & BaselineLen=  46.640 & BaselinePA=   11.986 & end
  "I1-L0": begin & BaselineLen=  51.237 & BaselinePA=   19.694 & end
  "I1-M0": begin & BaselineLen=  56.575 & BaselinePA=   26.027 & end
  "I1-U1": begin & BaselineLen= 113.700 & BaselinePA=  -69.696 & end
  "I1-U2": begin & BaselineLen= 121.850 & BaselinePA=  -42.184 & end
  "I1-U3": begin & BaselineLen= 136.205 & BaselinePA=  -22.351 & end
  "I1-U4": begin & BaselineLen= 103.997 & BaselinePA=    3.636 & end
  "J1-K0": begin & BaselineLen=  25.281 & BaselinePA=   -0.569 & end
  "J1-L0": begin & BaselineLen=  28.842 & BaselinePA=   14.721 & end
  "J1-M0": begin & BaselineLen=  33.933 & BaselinePA=   26.022 & end
  "J1-U1": begin & BaselineLen= 118.129 & BaselinePA=  -80.690 & end
  "J1-U2": begin & BaselineLen= 115.380 & BaselinePA=  -52.683 & end
  "J1-U3": begin & BaselineLen= 122.345 & BaselinePA=  -30.304 & end
  "J1-U4": begin & BaselineLen=  83.510 & BaselinePA=   -2.294 & end
  "J2-K0": begin & BaselineLen=  48.659 & BaselinePA=   -9.538 & end
  "J2-L0": begin & BaselineLen=  50.605 & BaselinePA=   -0.546 & end
  "J2-M0": begin & BaselineLen=  53.669 & BaselinePA=    7.576 & end
  "J2-U1": begin & BaselineLen= 131.225 & BaselinePA=  -71.418 & end
  "J2-U2": begin & BaselineLen= 136.012 & BaselinePA=  -47.061 & end
  "J2-U3": begin & BaselineLen= 145.967 & BaselinePA=  -28.453 & end
  "J2-U4": begin & BaselineLen= 106.735 & BaselinePA=   -5.998 & end
  "J3-K0": begin & BaselineLen=  56.569 & BaselinePA=  152.900 & end
  "J3-L0": begin & BaselineLen=  58.236 & BaselinePA=  145.064 & end
  "J3-M0": begin & BaselineLen=  60.921 & BaselinePA=  137.819 & end
  "J3-U1": begin & BaselineLen= 106.748 & BaselinePA= -121.975 & end
  "J3-U2": begin & BaselineLen=  65.986 & BaselinePA=  -94.948 & end
  "J3-U3": begin & BaselineLen=  46.635 & BaselinePA=  -49.978 & end
  "J3-U4": begin & BaselineLen=  23.984 & BaselinePA=   71.006 & end
  "J4-K0": begin & BaselineLen=  72.441 & BaselinePA=  154.687 & end
  "J4-L0": begin & BaselineLen=  73.749 & BaselinePA=  148.482 & end
  "J4-M0": begin & BaselineLen=  75.887 & BaselinePA=  142.582 & end
  "J4-U1": begin & BaselineLen= 111.440 & BaselinePA= -130.015 & end
  "J4-U2": begin & BaselineLen=  64.016 & BaselinePA= -108.978 & end
  "J4-U3": begin & BaselineLen=  33.937 & BaselinePA=  -64.026 & end
  "J4-U4": begin & BaselineLen=  28.827 & BaselinePA=  104.712 & end
  "J5-K0": begin & BaselineLen=  96.325 & BaselinePA=  156.260 & end
  "J5-L0": begin & BaselineLen=  97.310 & BaselinePA=  151.549 & end
  "J5-M0": begin & BaselineLen=  98.941 & BaselinePA=  146.980 & end
  "J5-U1": begin & BaselineLen= 122.122 & BaselinePA= -140.582 & end
  "J5-U2": begin & BaselineLen=  68.362 & BaselinePA= -129.525 & end
  "J5-U3": begin & BaselineLen=  24.015 & BaselinePA= -109.019 & end
  "J5-U4": begin & BaselineLen=  46.628 & BaselinePA=  130.061 & end
  "J6-K0": begin & BaselineLen= 120.261 & BaselinePA=  157.208 & end
  "J6-L0": begin & BaselineLen= 121.051 & BaselinePA=  153.418 & end
  "J6-M0": begin & BaselineLen= 122.366 & BaselinePA=  149.707 & end
  "J6-U1": begin & BaselineLen= 136.241 & BaselinePA= -149.213 & end
  "J6-U2": begin & BaselineLen=  80.007 & BaselinePA= -145.841 & end
  "J6-U3": begin & BaselineLen=  33.964 & BaselinePA= -153.987 & end
  "J6-U4": begin & BaselineLen=  68.337 & BaselinePA=  140.470 & end
  "K0-L0": begin & BaselineLen=   8.019 & BaselinePA=   70.959 & end
  "K0-M0": begin & BaselineLen=  16.011 & BaselinePA=   70.995 & end
  "K0-U1": begin & BaselineLen= 116.485 & BaselinePA=  -93.036 & end
  "K0-U2": begin & BaselineLen= 101.829 & BaselinePA=  -63.982 & end
  "K0-U3": begin & BaselineLen= 101.173 & BaselinePA=  -37.423 & end
  "K0-U4": begin & BaselineLen=  58.246 & BaselinePA=   -3.042 & end
  "L0-M0": begin & BaselineLen=   7.992 & BaselinePA=   71.030 & end
  "L0-U1": begin & BaselineLen= 124.213 & BaselinePA=  -94.056 & end
  "L0-U2": begin & BaselineLen= 107.643 & BaselinePA=  -67.005 & end
  "L0-U3": begin & BaselineLen= 103.980 & BaselinePA=  -41.620 & end
  "L0-U4": begin & BaselineLen=  56.564 & BaselinePA=  -10.875 & end
  "M0-U1": begin & BaselineLen= 131.952 & BaselinePA=  -94.949 & end
  "M0-U2": begin & BaselineLen= 113.711 & BaselinePA=  -69.699 & end
  "M0-U3": begin & BaselineLen= 107.312 & BaselinePA=  -45.561 & end
  "M0-U4": begin & BaselineLen=  56.000 & BaselinePA=  -18.997 & end
      else: message,"Unknown baseline "+stations
  endcase

  ; coordinates of baseline in horizontal system
  BPAr  = BaselinePA * !PI/180.
  BaseN = BaselineLen * cos(BPAr)
  BaseE = BaselineLen * sin(BPAr)


  ; rotate around y-axis into equatorial system
  r1 = BaseN * (-sin_la)          ; + c * cos_la
  r2 = BaseE
  r3 = BaseN * (-cos_la)

  ; rotate into star-centered system
  a_E =   r1 * sinH + r2 * cosH
  b_N = (-r1 * cosH + r2 * sinH) * sin(de) - r3 * cos(de)
  ; length along line of sight doesn't matter

  base_len = sqrt( a_E * a_E + b_N * b_N );
  base_pa  = atan(a_E, b_N) * 180./!PI;
end

FUNCTION calibrate_disk, filename, diameter, lambda
  ; filename = MIDI filename string (will use first file)
  ; diameter = diameter in mas
  ; lambda   = array of wavelengths in micron

  baseline, filename[0], LENGTH=base_len, ANGLE=base_pa, QUIET=(!MIAdebug eq 0)

  diarad = diameter/1000./3600./180. * !PI
  print, "Diameter ", diameter, "mas = ", diarad, "rad"

  x  = !pi * base_len * diarad / (lambda / 1e6)
  vis = abs(2. * beselj(x,1) / x)

  if !MIAdebug then begin
      print,"      lambda      x_bessel     visibility"
      for i=0,N_elements(x)-1 do print, lambda[i], x[i], vis[i]
  endif
  return, vis
end
