;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;;
;; reducenight.pro
;; Updated version of 'reducenight' by Frank Przygodda
;; Thorsten Ratzka, 4. August 2005, 18:09
;;

pro reducenight,dir,GRIS=gris,CALONLY=calonly,STEP=step,DISPERSED=dispersed

  if ( N_ELEMENTS(dir) eq 0) then begin
	print,"usage: reducenight,'DIR',[/GRIS],[/CALONLY],[/STEP],[/DISPERSED]"
        print,"Call procedure from local directory where files can be stored."
        print,"-------------------------------------------------------------------"
	print,"Script looks for fringe tracking files in DIR and reduces data"
	print,"with xmidispvisi. Calibrator datasets are used to calibrate object"
	print,"datasets. The average visibility of each object with errorbars is"
	print,"calculated."
        print,"-------------------------------------------------------------------"
        print,"Default: PRISM reduction, give option for GRISM reduction."
        print,"Default: Fourier tracking, give option for DISPERSED reduction."
        print,"CALONLY -> Check only the instrumental visibilities"
        print,"STEP -> Sample visibility with maximum rate"
	return
  endif

;------------------------

  disp = 'PRIS'
  IF keyword_set(gris) THEN disp = 'GRIS'


  print,''
  print,'### Step 1) Looking for fringtrack files and creating file lists ...'
  print,'### Criterium: '+disp
  print,''

  IF strmid(dir,strlen(dir)-1) NE '/' THEN dir=dir+'/'

  spawn,'test -f logbook.txt && echo file exists',logexists
  IF logexists[0] EQ '' THEN  make_midi_log,dir $
                  ELSE print,'### Using already created logbook.txt file...'

  spawn,"cat logbook.txt | grep 'OBS_FRINGE_TRAC' | grep "+disp+" | tee logbook_selection.txt ",selection

  spawn,"cat logbook.txt | grep 'DEFAULT_CHOP' | grep "+disp+" | tee logbook_photfiles.txt ",photsel

  print,'### Resulting file list:'

  print,'TRACK FILES:'

  print,selection
  print,''
  print,'PHOT FILES:'
  print,photsel

  nsel = n_elements(selection)
  selist = strarr(10,nsel)
  FOR i=0,nsel-1 DO selist[*,i] =strsplit(selection[i],/extract)

  nphot = n_elements(photsel)
  photlist = strarr(10,nphot)
  FOR i=0,nphot-1 DO photlist[*,i] =strsplit(photsel[i],/extract)



;------------------------

  limit = 3.5  ; minutes

  print,''
  print,'### Step 2) Looking for files belonging together'
  print,'### Criterium: Following file observed within '+strtrim(string(limit),2)+' minutes'
  print,'### Result: see diagram'

  ;;; first the trackfiles ...

  day = float(strmid(selist[0,0],8,2))
  obstime = fltarr(nsel)

  FOR i=0,nsel-1 DO BEGIN
     obstime[i] = float(strmid(selist[0,i],11,2)) $
               + float(strmid(selist[0,i],14,2)) /60.$
	      + float(strmid(selist[0,i],17,6)) /(60.*60.)
     IF float(strmid(selist[0,i],8,2)) gt day THEN obstime[i] = obstime[i] +24.
  END
  y = indgen(nsel)
  xmin = min(obstime[*])-2.5
  xmax = max(obstime[*])+2.5

  siz = get_screen_size()
  siz[0] =   siz[0]*0.8
  set_plot,'x'
  window,xsize=siz[0],ysize=siz[1]
  !P.MULTI = [0, 1, 2]

  charx = !D.x_ch_size/siz(0)*(xmax-xmin)
  chary = !D.y_ch_size/siz(1)*nsel


  plot,obstime[*],y,psym=6,xtitle='TIME (UT)',ytitle='FILE',xrange=[xmin,xmax],yrange=[-2,nsel+1],title = 'FRINGE TRACK FILES ('+disp+') in '+dir

  trackfiles = strarr(nsel)
  plotlinex = fltarr(nsel,2)
  plotliney = fltarr(nsel,2)
  name = strarr(nsel)
  file = strarr(nsel)
  diameter = strarr(nsel)
  chopfreq = fltarr(nsel)
  phota = strarr(nsel)
  photb = strarr(nsel)

  pre = dir+'MIDI.'
  suf= '.fits'

  trackfiles[0] = pre+selist[0,0]+suf
  plotlinex[0,0] = obstime[0]
  plotliney[0,0] = y[0]
  name[0] = selist[1,0]
  file[0] = selist[0,0]
  diameter[0] = (calibrator(name[0])).diam
  chopfreq[0] = 0.0
  ;chopfreq[0] = float(selist[9,0])
  phota[0] = '?'
  photb[0] = '?'
  n  = 0
  FOR i=1,nsel-1 DO BEGIN
    IF (obstime[i]-obstime[i-1])*60. lt limit  THEN  BEGIN
         trackfiles[n]= trackfiles[n]+' '+pre+selist[0,i]+suf
         plotlinex[n,1] = obstime[i]
         plotliney[n,1] = y[i]
    ENDIF ELSE BEGIN
         n=n+1
         trackfiles[n] = pre+selist[0,i]+suf
         plotlinex[n,0] = obstime[i]
         plotliney[n,0] = y[i]
         name[n] = selist[1,i]
         file[n] = selist[0,i]
         diameter[n] = (calibrator(name[n])).diam
         chopfreq[n] = 0.0
         ;chopfreq[n] = float(selist[9,i])
         phota[n] = '?'
         photb[n] = '?'
    ENDELSE
  ENDFOR
  ntracks = n+1

  ; create diagram
  FOR i=0,ntracks-1 DO BEGIN
    IF plotlinex[i,1] NE 0 THEN BEGIN
        IF chopfreq[i] EQ 0.0 THEN chopping = '  (needs PHOTFILE)' ELSE  chopping = ''
        oplot,plotlinex[i,*],plotliney[i,*]
        xyouts,plotlinex[i,0]-charx*10,plotliney[i,0]-chary*.5,'TRACK '+strtrim(string(i),2)
        xyouts,plotlinex[i,0]+charx*3,plotliney[i,0]-chary*.5,name[i]+chopping,color=200
    ENDIF ELSE BEGIN
        IF chopfreq[i] EQ 0.0 THEN chopping = '  (needs PHOTFILE)' ELSE  chopping = ''
        xyouts,plotlinex[i,0]-charx*10,plotliney[i,0]-chary*.5,'TRACK '+strtrim(string(i),2)
        xyouts,plotlinex[i,0]+charx*3,plotliney[i,0]-chary*.5,name[i]+chopping,color=200
    ENDELSE
  ENDFOR

  ;;; now the photfiles ...

  charx = !D.x_ch_size/siz(0)*(xmax-xmin)
  chary = !D.y_ch_size/siz(1)*nphot

  day = float(strmid(photlist[0,0],8,2))
  obstime = fltarr(nphot)

  FOR i=0,nphot-1 DO BEGIN
     obstime[i] = float(strmid(photlist[0,i],11,2)) $
               + float(strmid(photlist[0,i],14,2)) /60.$
	      + float(strmid(photlist[0,i],17,6)) /(60.*60.)
     IF float(strmid(photlist[0,i],8,2)) gt day THEN obstime[i] = obstime[i] +24.

  END
  y = indgen(nphot)

  plot,obstime[*],y,psym=6,xtitle='TIME (UT)',ytitle='FILE',xrange=[xmin,xmax],yrange=[-2,nphot+1],title = 'PHOTOMETRIC FILES ('+disp+') in '+dir

  photfiles = strarr(nphot)
  plotlinex = fltarr(nphot,2)
  plotliney = fltarr(nphot,2)
  photname = strarr(nphot)
  photfile = strarr(nphot)
  abopen = strarr(nphot)

  pre = dir+'MIDI.'
  suf= '.fits'

  photfiles[0] = pre+photlist[0,0]+suf
  plotlinex[0,0] = obstime[0]
  plotliney[0,0] = y[0]
  photname[0] = photlist[1,0]
  photfile[0] = photlist[0,0]
  abopen[0] = photlist[5,0]
  n  = 0
  FOR i=1,nphot-1 DO BEGIN
     IF ((obstime[i]-obstime[i-1])*60. lt limit)  AND (photlist[5,i] EQ photlist[5,i-1]) THEN  BEGIN
         photfiles[n]= photfiles[n]+' '+pre+photlist[0,i]+suf
         plotlinex[n,1] = obstime[i]
         plotliney[n,1] = y[i]
    ENDIF ELSE BEGIN
         n=n+1
         photfiles[n] = pre+photlist[0,i]+suf
         plotlinex[n,0] = obstime[i]
         plotliney[n,0] = y[i]
         photname[n] = photlist[1,i]
         photfile[n] = photlist[0,i]
         abopen[n] = photlist[5,i]
    ENDELSE
  ENDFOR
  nphots = n+1

  ;;; create diagram
  FOR i=0,nphots-1 DO BEGIN
    IF plotlinex[i,1] NE 0 THEN BEGIN
        oplot,plotlinex[i,*],plotliney[i,*]
        xyouts,plotlinex[i,0]-charx*12,plotliney[i,0]-chary*.5,'PHOT_'+strmid(abopen(i),0,1)+' '+strtrim(string(i),2)
        xyouts,plotlinex[i,0]+charx*3,plotliney[i,0]-chary*.5,photname[i],color=200
    ENDIF ELSE BEGIN
        xyouts,plotlinex[i,0]-charx*12,plotliney[i,0]-chary*.5,'PHOT_'+strmid(abopen(i),0,1)+' '+strtrim(string(i),2)
        xyouts,plotlinex[i,0]+charx*3,plotliney[i,0]-chary*.5,photname[i],color=200
    ENDELSE
  ENDFOR

  print,'### Saving trackslist ...'
  spawn,'test -f tracklist.txt && echo file exists',listexists
  ok = ''
  IF listexists[0] NE '' THEN BEGIN
     print,'### Warning: found old tracklist on disk (tracklist.txt), overwrite?'
     read,ok
  ENDIF
  IF ok EQ 'y' or ok EQ 'yes' or listexists[0] EQ '' THEN BEGIN
     openw, luna, 'tracklist.txt', /Get_LUN
     printf,luna,"# track list for ",dir
     printf,luna,"# nr. files                         object   diameter  photfile A B"
     FOR i=0,ntracks-1 DO BEGIN
         IF chopfreq[i] NE 0.0 THEN BEGIN
             phota[i] = '-'
             photb[i] = '-'
         ENDIF
         printf,luna,i,file[i],n_elements(strsplit(trackfiles[i],' ',/extract))-1,name[i],$
           diameter[i],phota[i],photb[i],  FORMAT = '(I3," ",A26,"+",I1,A12, F8.2,"      ",A3,A3 )'
     ENDFOR
         close,luna
  ENDIF


;------------------------

  print,''
  print,'### Step 3) Reducing fringe track data with xmidispvisi'
  print,''

  print,'### Edit tracklist?'
  editlist = ''
  read,editlist

  IF editlist EQ 'y' OR editlist EQ 'yes' THEN BEGIN
      print,'### Please EDIT tracklist so that it contains only tracks to reduce'
      print,'### (Erase lines or comment with #-symbol at begining)'
      print,'### Replace "? ?" with numbers of photometric files A and B'
      print,'### (Sci-phot-mode: no phot files necessary, indicated by "-  -")'
      print,'### To treat calibrator as an object set its diameter to -1'
      print,'### CLOSE editor to go on with data reduction'

      editor=""
      mess=""
      errmess=""

      editor = getenv("VISUAL")
      if editor eq "" then editor = "emacs"
      spawn, editor+' tracklist.txt',mess,errmess

      while (errmess eq editor+": Command not found.") do begin
          print,''
	  print,'WARNING!!! I need your help, because I cannot find "'+editor+'"!'
          read,"Please enter the name of your favourite editor: ",editor
          print,''
	  spawn, editor+' tracklist.txt',mess,errmess
      endwhile

  ENDIF

  ;;; read tracklist
  spawn,'cat tracklist.txt | sed "/^#/d" | sed -n "/^.\{40\}/p"',tracklist_tmp
  ntracks = n_elements(tracklist_tmp)
  tracklist = strarr(6,ntracks)
  FOR i=0,ntracks-1 DO tracklist[*,i] =strsplit(tracklist_tmp[i],/extract)


  ;;; REDUCTION LOOP

  FOR i=0,ntracks-1 DO BEGIN
    t = fix(tracklist[0,i])
    print,''
    print,'### TRACK '+ strtrim(string(t),2)+':'

    spawn,'test -f Track'+strtrim(string(t),2)+'.sav  && echo file exists',trackexists

    skip = ''
    IF trackexists[0] NE '' THEN BEGIN
       print,'### Found saved reduction file (Track'+strtrim(string(t),2)+'.sav), Skip reduction?'
       read,skip
    ENDIF

    tocheck = 0
    IF skip EQ 'n' OR skip EQ 'no' OR trackexists[0] EQ '' THEN BEGIN
        IF tracklist[4,i] EQ '-' THEN BEGIN
         print,'### Track Files: '+trackfiles[t]
         IF KEYWORD_SET(DISPERSED) $
                THEN  x = obj_new('xmidispvisi',trackfiles[t],dispersedtrack=50) $
                ELSE  x = obj_new('xmidispvisi',trackfiles[t])
      ENDIF ELSE BEGIN
         photafiles = photfiles(fix(tracklist(4,where(t EQ fix(tracklist[0,*])))))
         photbfiles = photfiles(fix(tracklist(5,where(t EQ fix(tracklist[0,*])))))
         print,'### Track Files: '+trackfiles[t]
         print,'### Phot Files A: '+photafiles
         print,'### Phot Files B: '+photbfiles
         ;chop_nod_disp,photafiles
         ;chop_nod_disp,photbfiles
        IF KEYWORD_SET(DISPERSED) $
           THEN  x = obj_new('xmidispvisi',trackfiles[t], photafiles, photbfiles,dispersedtrack=50) $
           ELSE  x = obj_new('xmidispvisi',trackfiles[t], photafiles, photbfiles)
      ENDELSE
      tocheck = 1
    ENDIF

    check = ''
    IF tocheck EQ 0 THEN BEGIN
        print,'### Quality check of fringetrack with gui?'
        read,check
    ENDIF

    IF check EQ 'y' OR check EQ 'yes' OR tocheck EQ 1  THEN BEGIN

      ;;; read, if it is not a new track ...
      IF tocheck NE 1 THEN restore,'Track'+strtrim(string(t),2)+'.sav'



      print,'### Displaying Track '+strtrim(string(t),2)+'. ADJUST thresholds then CLOSE gui ...'
      x-> gui,/BLOCK,/TRUECOLOR

      ;;;; saving object ...
      im = obj_new('fitsfile',strmid(trackfiles[t],0,strpos(trackfiles[t],' ')))
      he = im->prihead()
      OBJECT        = STRCOMPRESS(he->getpar('OBS TARG NAME'),/remove_all)
      obj_destroy,im
      name = 'Track'+strtrim(string(t),2)+'.sav'
      save,x,filename=name
      print,'### saved object file: '+name
      obj_destroy,x
    ENDIF

ENDFOR ; reduction loop


;------------------------

  if not keyword_set(CALONLY) then begin

  TVLCT,[0,150,0,0,0,150,150],[0,0,150,0,150,0,150],[0,0,0,150,150,150,0]

  print,''
  print,'### Step 4) Calibrating each object with all calibrators'

  diameter=float(tracklist[3,*])
  nobj= n_elements(where(diameter EQ -1.0))
  ncal= n_elements(where(diameter NE -1.0))

  allvisi = fltarr(ncal,3,320); big array for all calibrated visibilities:
  siz = get_screen_size()
  siz[0] =   siz[0]*0.8
  ;set_plot,'x'
  ;window,xsize=siz[0],ysize=siz[1]
  ;!P.MULTI = [0, 1, 2]

  FOR i = 0,nobj-1 DO BEGIN
       objtrack_tmp = tracklist[0,where(diameter EQ -1.0) ]
       objtrack = fix(objtrack_tmp[i])
       objtrack_tmp = tracklist[2,where(diameter EQ -1.0) ]
       objname = objtrack_tmp[i]
       objfile_tmp =  tracklist[1,where(diameter EQ -1.0) ]
       objfile =  objfile_tmp[i]

       set_plot,'PS'
       DEVICE,FILE=objname+'.ps',/COLOR

       if ncal gt 1 then !P.MULTI = [0, 1, 2] else !P.MULTI = [0, 1, 1]
       if ncal gt 1 then poststring='ies' else poststring='y'

       FOR j = 0,ncal-1 DO BEGIN

       caltrack_tmp = tracklist[0,where(diameter NE -1.0) ]
       caltrack = fix(caltrack_tmp[j])
       caltrack_tmp = tracklist[2,where(diameter NE -1.0) ]
       calname = caltrack_tmp[j]
       caltrack_tmp = tracklist[3,where(diameter NE -1.0) ]
       caldia = caltrack_tmp[j]

       print,'### Calibrating TRACK '+string(strtrim(objtrack,2))+' ('+objname+') with TRACK '+string(strtrim(caltrack,2))+' ('+calname+')'

       ;;; load track files
       restore,'Track'+string(strtrim(caltrack,2))+'.sav'
       c = x
       restore,'Track'+string(strtrim(objtrack,2))+'.sav'

       ;;; set the binning

       if keyword_set (STEP) then  x-> set_lambda_bins, 41, 129, 1, 4

       ;;; calibrate
       calvis = x->calibratedvisi(c,caldia)
       bins = n_elements(calvis[0,*])

       openw,luna,objname+'_Track'+string(strtrim(objtrack,2))+'+'+string(strtrim(caltrack,2))+'_calvis.dat', /Get_LUN
       FOR n=0,bins-1 DO printf,luna,calvis[0,n],calvis[1,n],calvis[2,n]
       close, luna

       IF j EQ 0 THEN BEGIN
            plot,calvis[0,*],calvis[2,*],yrange=[0,1], xmargin=[6,1], ymargin=[3,0],$
	    xtitle='Wavelength [!4l!Xm]', ytitle='Calibrated Visibilit'+poststring,$
	    color=j, linestyle=0;,title=objname+' (Track '+string(strtrim(objtrack,2))+')'
       ENDIF ELSE BEGIN
            oplot,calvis[0,*],calvis[2,*],color=j
       ENDELSE

       allvisi[j,0,0:bins-1] = calvis[0,*]
       allvisi[j,1,0:bins-1] = calvis[1,*]
       allvisi[j,2,0:bins-1] = calvis[2,*]

       obj_destroy,c
       obj_destroy,x

   ENDFOR  ; 'each calibrator' loop

  ; Plot result


       allmean = total(allvisi[*,*,*],1)/ncal

     if ncal gt 1 then begin

       plot,allmean[0,0:bins-1],allmean[2,0:bins-1],yrange=[0,1], xmargin=[6,1], ymargin=[3,1],$
	      xtitle='Wavelength [!4l!Xm]', ytitle='Mean Calibrated Visibility',$
	      color=0, linestyle=0;,title=objname+' (File '+objfile+') with '+strtrim(ncal,2)+' Cal.'

     endif

     IF ncal NE 1 THEN BEGIN
        allstd = fltarr(2,320)
        allstd[0,*] = allmean[0,*]
        FOR n=0,bins-1  DO allstd[1,n]  = stddev(allvisi[*,2,n])

       if ncal gt 1 then begin
         errplot,allmean[0,0:bins-1],allmean[2,0:bins-1]+allstd[1,0:bins-1],allmean[2,0:bins-1]-allstd[1,0:bins-1]
       endif

     ENDIF ELSE  allstd = allmean * 0.0

     DEVICE,/CLOSE
     SET_PLOT,'X'

     openw, luna,objname+'_Track'+string(strtrim(objtrack,2))+'_resultvis.dat',Get_LUN
     FOR n=0,bins-1 DO printf,luna,allmean[0,n],allmean[1,n],allmean[2,n],allstd[1,n]
     close, luna

     set_plot,"ps"
     !P.MULTI = [0, 1, 1]
     device,file=objname+'_Track'+string(strtrim(objtrack,2))+'_resultvis.ps',/portrait,/COLOR
     plot,allmean[0,0:bins-1],allmean[2,0:bins-1],yrange=[0,1],title=objname+' (File '+objfile+') with '+strtrim(ncal,2)+' Cal.',xtitle='micron',ytitle='V'
     IF total(allstd[1,0:bins-1]) NE 0.0 THEN $
       errplot,allmean[0,0:bins-1],allmean[2,0:bins-1]+allstd[1,0:bins-1],allmean[2,0:bins-1]-allstd[1,0:bins-1]
     device,/close
     set_plot,'x'
     !P.MULTI = [0, 1, 2]

     ;IF j NE ncal-1 THEN BEGIN
     ;  print,''
     ;  print,'### Enjoy visibility then press return to go to next object file...'
     ;  read,ok
     ;ENDIF

  ENDFOR                          ;  'each object' loop


  print,''
  print,'### Results saved in following files:'

  spawn,'ls -1tr *vis.dat'
  spawn,'ls -1tr *vis.ps'

  endif

;------------------------

  print,''
  print,'### Step 5) Check the instrumental Visibility'

  TVLCT,[0,150,0,0,0,150,150],[0,0,150,0,150,0,150],[0,0,0,150,150,150,0]

  diameter=float(tracklist[3,*])
  ncal= n_elements(where(diameter NE -1.0))

print,ncal
  allvisi = fltarr(ncal,3,320); big array for all instrumental visibilities:
  siz = get_screen_size()
  siz[0] =   siz[0]*0.8
  SET_PLOT,'PS'
  DEVICE,FILE='Cal.ps',/PORTRAIT,/COLOR
  !P.MULTI = [0, 1, 1]

  FOR j = 0,ncal-1 DO BEGIN

       if ncal gt 1 then poststring='ies' else poststring='y'

       caltrack_tmp = tracklist[0,where(diameter NE -1.0) ]
       caltrack = fix(caltrack_tmp[j])
       caltrack_tmp = tracklist[2,where(diameter NE -1.0) ]
       calname = caltrack_tmp[j]
       caltrack_tmp = tracklist[3,where(diameter NE -1.0) ]
       caldia = caltrack_tmp[j]

       print,'### Instrumental visibility of TRACK '+string(strtrim(caltrack,2))+' ('+calname+')'

       ;;; load track files
       restore,'Track'+string(strtrim(caltrack,2))+'.sav'
       c = x

       ;;; calibrate
       visibi = c -> visibility()
       insvis = c->instruvisi()
       bins = n_elements(insvis[0,*])

       IF j EQ 0 THEN BEGIN
            plot,insvis[0,*],insvis[2,*],yrange=[0.5,1], xmargin=[6,1], ymargin=[3,0],$
	    xtitle='Wavelength [!4l!Xm]', ytitle='Instrumental Visibilit'+poststring,$
	    color=j, linestyle=0
            oplot,visibi[0,*],visibi[2,*],color=j, linestyle=2
       ENDIF ELSE BEGIN
            oplot,insvis[0,*],insvis[2,*],color=j, linestyle=0
            oplot,visibi[0,*],visibi[2,*],color=j, linestyle=2
       ENDELSE

       if ncal gt 1 then begin
         oplot, [8.2,8.5],[0.515+ncal*0.02-j*0.02,0.515+ncal*0.02-j*0.02],color=j,linestyle=0
         xyouts,8.6,0.51+ncal*0.02-j*0.02,calname+' ('+caldia+' mas) '
       endif

       allvisi[j,0,0:bins-1] = insvis[0,*]
       allvisi[j,1,0:bins-1] = insvis[1,*]
       allvisi[j,2,0:bins-1] = insvis[2,*]

       obj_destroy,c
       obj_destroy,x

   ENDFOR  ; 'instrumental visibility' loop

   DEVICE,/CLOSE
   SET_PLOT,'X'
end
