/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: VLTIfileUtil.h,v 1.5 2004/02/12 13:25:57 bcotton Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
/* $Id: VLTIfileUtil.h,v 1.5 2004/02/12 13:25:57 bcotton Exp $ */

#ifndef _VLTIFILEUTIL_H
#define _VLTIFILEUTIL_H
/*----------------------------------------------------------------------- */
/* FITS file utility routines                                             */
/*----------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "VLTItable.h"

int append_tables(char *infile, char *outfile);
/*----------------------------------------------------------------------- */
/*  Append all tables from one file onto another.                         */
/*  This is done by copying all records following the primary HDU from    */
/*  the input to the output.                                              */
/*   Inputs:                                                              */
/*      infile    input file name                                         */
/*      outfile   output file name                                        */
/*   Returns:                                                             */
/*      0 on success, else error                                          */
/*----------------------------------------------------------------------- */

void copy_table(char *table_type, fitsfile *infptr, fitsfile *outfptr, 
		int *ierr);
/*----------------------------------------------------------------------- */
/*  Copy a specified table from one open FITS file to another             */
/*   Inputs:                                                              */
/*      table_type table type name                                        */
/*      infptr     input file cfitsio structure                           */
/*      outfptr    output file  cfitsio structure                         */
/*   Outputs:                                                             */
/*      ierr       Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

void copy_header_keywords(fitsfile *infptr, fitsfile *outfptr, 
			  int nkeys, char** keys, int *ierr);
/*----------------------------------------------------------------------- */
/*  Copy all keywords in a specified list from one header to another.     */
/*  ESO Heirarchal keywords and specified list are copied.                */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      outfptr    output file  cfitsio structure                         */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      ierr       Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

int default_fits_read_key(fitsfile *infptr, int datatype, 
			  char *keyname, void* value, char *comment, 
			  void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a given keyword but if it is not present return a     */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      datatype   cfitsio data type code                                 */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

int default_fits_read_key_str(fitsfile *infptr, 
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a string keyword but if it is not present return a    */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

int default_fits_read_key_dbl(fitsfile *infptr, 
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a double keyword but if it is not present return a    */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

int default_fits_read_key_flt(fitsfile *infptr, 
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a float keyword but if it is not present return a     */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

int default_fits_read_key_lng(fitsfile *infptr, 
			      char *keyname, void* value, char *comment, 
			      void* deflt, int *status);
/*----------------------------------------------------------------------- */
/*  Attempt to read a long keyword but if it is not present return a      */
/*  default value.                                                        */
/*   Inputs:                                                              */
/*      infptr     input file cfitsio structure                           */
/*      keyname    keyword                                                */
/*      deflt      pointer to default value                               */
/*      nkeys      number of entries in keys                              */
/*      keys       array of keywords to copy                              */
/*   Outputs:                                                             */
/*      value      pointer to value                                       */
/*      comment    comment on keyword                                     */
/*      status     Error code, 0 = OK, else failed, checked on input      */
/*----------------------------------------------------------------------- */

/*  Return a NaN  */
float give_float_NaN();
/*----------------------------------------------------------------------- */
/*  return a float NaN (not a number)                                     */
/*  uses a word with all bits on                                          */
/*----------------------------------------------------------------------- */

#endif /* end VLTIFILEUTIL_H */
/* */
