;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
FUNCTION STRtemplate, st, tmp
;        insert string st into tmp without changing its length
   l = STRLEN(tmp)
   t = STRING(REPLICATE(32B,l))
   STRPUT,t,st
RETURN, t
END


PRO assembleDatabase, prefix
   print,'     Bands files',format='(a,$)'
   bands = OBJ_NEW('bands',prefix+'Bands.fits',/table)
   bands->close
   bands->newFile,prefix+'Database.fits',/table
   tableFile = bands->file()
   tableFile->noClose
   bands->detachFromFile
   print,':Done'

;      get bandpass tables ; first find out how many
   bfile    = OBJ_NEW('fitsfile',prefix+'BandPasses.fits')
   bext     = bfile->listExtensions()
   nBands   = N_ELEMENTS(bext)
   OBJ_DESTROY,bfile
   print,nBands,' Bandpass files',format='(i4,a,$)'
   for i=1, nBands do begin
      bandpass = OBJ_NEW('bandpass',prefix+'BandPasses.fits',extnu=i,/table)
      bandpass->close
      bandpass->appendToFile,tableFile,/table
      bandpass->detachFromFile
      OBJ_DESTROY,bandpass
   endfor
   print,':Done'
   print,'     Reference files',format='(a,$)'
   refs  = OBJ_NEW('references',prefix+'References.fits',/table)
   refs->close
   refs->appendToFile,tableFile,/table
   refs->detachFromFile
   OBJ_DESTROY,refs
   print,':Done'

   print,'     Alias files',format='(a,$)'
   aliases = OBJ_NEW('aliases',prefix+'Aliases.fits',/table)
   aliases->close
   aliases->appendToFile,tableFile,/table
   aliases->detachFromFile
   OBJ_DESTROY,aliases
   print,':Done'

   print,'     Sources files',format='(a,$)'
   sources = OBJ_NEW('sources',prefix+'Sources.fits',/table)
   sources->close
   sources->appendToFile,tableFile,/table
   tSources = *(sources->table()) ; get hands on table 
   sources->detachFromFile
   OBJ_DESTROY,sources
   print,':Done'
   nSources = N_ELEMENTS(tSources)
  
   print,nSources,' Spectrophotometry files',format='(i4,a,$)'
   for i=0,nSources-1 do begin
      star = STRTRIM(tSources[i].name,2)
      sp = OBJ_NEW('spectrophotometry',prefix+star+'Photometry.fits',/table)
      sp->close
      sp->appendToFile,tableFile,/table
      sp->detachFromFile
      OBJ_DESTROY,sp
      if (0 eq (i mod 20)) then print,i,format='(i4,$)'
   endfor
   print,':Done'
;
   print,'     Diameter files',format='(a,$)'
   diameters = OBJ_NEW('diameters',prefix+'Diameters.fits',/table)
   diameters->close
   diameters->appendToFile,tableFile,/table
   diameters->detachFromFile
   OBJ_DESTROY,diameters
   print,':Done'
;
   print,'     Photometry files',format='(a,$)'
   photometry = OBJ_NEW('photometry',prefix+'Photometry.fits',/table)
   photometry->close
   photometry->appendToFile,tableFile,/table
   tableFile->closeOK
   photometry->detachFromFile
   OBJ_DESTROY,photometry
   print,':Done'
;
   OBJ_DESTROY,tableFile
RETURN
END

FUNCTION databaseIndex, dbFile, ierr=ierr
;           assemble table information from a cal database
   tableData = {seqNo:0, type:'', name:'', band:'',modifier:'',nrows:0}
;
   inFile = OBJ_NEW('fitsfile', dbFile, iErr=iErr)
   if (iErr NE 0) then begin
      midiprinterrmsg
      RETURN,0
   endif

   tableList       = tableData
   inFile->rewind
   inFile->skipExt ; primary data section
   iErr = 0
   i = 0
   while(iErr EQ 0) do begin
      head  = inFile->extHead(ierr=ierr)
      if (ierr EQ 4) then goto, ateof
      nRows = head->getPar('NAXIS2')
      tableData.seqNo = i+1
      tableData.nRows = nRows
      tableData.type  = STRTRIM(head->getPar('EXTNAME'),2)
      tableData.band  = ''
      tableData.modifier  = ''
      CASE tableData.type of
         'BANDS': begin
         endcase
         'BANDPASS': begin
            tableData.band     = head->getPar('BAND')
            tableData.modifier = head->getPar('MODIFIER')
         endcase
         'ALIASES': begin
         endcase
         'REFERENCES': begin
         endcase
         'SOURCES': begin
         endcase
         'SPECTROPHOTOMETRY': begin
            tableData.band     = head->getPar('BAND')
            tableData.modifier = head->getPar('MODIFIER')
            tableData.name   = head->getPar('NAME')
         endcase
         'PHOTOMETRY': begin
         endcase
         else: print,'  table type '+tableData.type+' unknown '
      ENDCASE
      OBJ_DESTROY,head
      if (i EQ 0) then tableList = tableData else $
         tableList = [tableList,tableData]
      inFile->skipext,ierr=ierr,/noeof
      i = i+1
   endWhile
ateof: OBJ_DESTROY,inFile
RETURN, tableList
END

PRO printDatabaseIndex, inFile, outFile
   tables = databaseIndex(infile, ierr=ierr)
   outputFile = N_PARAMS() GT 1
   if (ierr EQ 0) then RETURN

   nTables  = N_ELEMENTS(tables)

   if outputFile then OPENW, unit, outFile,/GET_LUN else unit = -1
   fmt = "(i5,' ',a24,'  ',i5,'  ',a16,'  ',a8,'  ',a16)"
   printf,unit,'Seq','Type','nRows','Source','Band','Band Modifier',$
      format="(a5,' ',a24,'  ',a5,'  ',a16,'  ',a8,'  ',a16)"
   for i=0,nTables-1 do printf,unit,tables[i].seqNo, $
      tables[i].type, tables[i].nrows, tables[i].name, $
      tables[i].band, tables[i].modifier,format=fmt
    if outputFile then FREE_LUN,unit
RETURN
END

FUNCTION vboekelbase
   db =  OBJ_NEW('caldatabase',getenv('calbase')+'/databases/vBoekelDatabase.fits')
RETURN,db
END
