;
; installwiz.pro
; Created:     Mon Sep 11 16:42:35 2006 by Koehler@kaag
; Last change: Mon Feb 26 14:18:27 2007
;
; PURPOSE:
;	install MIA+EWS without confusing the user
;
; COMPILING SEQUENCE:
;	start idl
;	.run installwiz.pro
;	save,/routines,file="installwiz.sav"
;
; CALLING SEQUENCE:
;	idl -vm=installwiz.sav
;
; INPUT:
;	config-parameters from ~/.mia+ews-config, if it exists
;
; OUTPUT:
;	compiles cfitsio and C-routines,
;	creates startup-files,
;	installs mia and mia.sh in dirs chosen by the user
;	writes config-parameters to ~/.mia+ews-config
;
PRO installwiz_event, ev
  Common PathDirs, pdirs

  ;;print, "Squeak!"
  ;;help,ev,/str

  if ev.tag eq "GO" then begin
      ;;print,"squeaksqueak"

      widget_control, ev.ID, get_value=v, /hourglass
      ;;help,v,/str
      bindir= pdirs[v.path]

      openw, cfg, "~/.mia+ews-config", /get_lun
      printf,cfg, 'make="'+v.make+'"'
      printf,cfg, 'browser="'+v.browser+'"'
      printf,cfg, 'bindir="'+bindir+'"'
      close, cfg
      free_lun,cfg

      spawn, "echo '----------------------------------------------------------------------' >> install.log",/sh
      spawn, "date >> install.log"
      spawn, "uname -a >> install.log"

      stage= "install_cfitsio"
      stat = widget_label(ev.ID, /align_left, value="Making cfitsio...")
      spawn, v.make+" install_cfitsio >> install.log 2>&1", /sh, Exit_status=status
      if status eq 0 then begin
          stage= "install_c"
          stat = widget_label(ev.ID, /align_left, value="Making C-programs...")
          spawn, v.make+" install_c >> install.log 2>&1", /sh, Exit_status=status
          if status eq 0 then begin
              stage= "startup-files"
              stat = widget_label(ev.ID, /align_left, value="Creating startup-files...")

              cd, current=cwd
              cmds= ['sed -e "s,%HERE%,'+cwd+'," -e "s,%BROWSER%,'+v.browser+'," STARTUP.in > STARTUP',$
                     'sed -e "s,%HERE%,'+cwd+'," setup.csh_in > setup.csh',$
                     'sed -e "s,%HERE%,'+cwd+'," mia.in > mia.sh',$
                     'cp -p mia.sh mia',$
                     'chmod a+x mia.sh mia' ]

              if bindir ne "No directory" then begin
                  cmds = [ cmds, $
                           'ln -sf '+cwd+'/mia.sh '+bindir+'/mia',$
                           'ln -sf '+cwd+'/mia.sh '+bindir+'/mia.sh' ]
              endif

              for i=0,N_elements(cmds)-1 do begin
                  spawn, "echo '"+cmds[i]+"' >> install.log"
                  spawn, cmds[i] + " 2>>install.log", /sh, Exit_status=status
                  if status ne 0 then break
              endfor
          endif
      endif
      if status ne 0 then begin
          msg= dialog_message(["Installation failed!",$
                               'Please check the file "install.log" or use',$
                               'make cl_install to figure out what went wrong.'$
                              ], /ERROR, Dialog_Parent=ev.top)
          print,"Installation failed!"
      endif else begin
          msg= dialog_message(["Installation successful!",$
                               "To start MIA+EWS, type",$
                               "",$
                               '  "mia" or "mia.sh"',$
                               '' ],$
                              /Information, Dialog_Parent=ev.top)
      endelse
  endif

  if ev.tag eq "GO" or ev.tag eq "CANCEL" then Widget_control, ev.top, /Destroy
end


PRO installwiz
  Common PathDirs, pdirs
  ;;
  ;; load a wizzard
  ;;
  read_jpeg, "installwiz.jpg", iwiz, /true
  ;;
  ;; find installation path (i.e. here)
  ;;
  cd, current=cwd
  ;;
  ;; figure out which make we have
  ;;
  ;;if !Version.os eq 'linux' then make="make" else make="gmake"
  spawn,"gmake -h > /dev/null",Exit_status=status
  ;;print,"make-status "+string(status)
  if status eq 0 then make="gmake" else make="make"
  ;;
  ;; default browser
  ;;
  browser = 'firefox'
  ;;
  ;; find potential dirs for executable (in path, writable, and not ".")
  ;;
  pdirs= strsplit(getenv("PATH"),":",/extract)
  widx = where(pdirs ne "." and file_test(pdirs,/write) eq 1)
  if widx[0] le -1 then pdirs= getenv("HOME")  else  pdirs= pdirs[widx]
  pdirs = [ "No directory", pdirs ]
  bidx = 0
  ;;
  ;; and now see what we got last time
  ;;
  if file_test("~/.mia+ews-config",/read) then begin
      openr, cfg, "~/.mia+ews-config", /get_lun
      line = ""
      while not eof(cfg) do begin
          readf,cfg,line
          ;;print,"config line = ",line
          r = execute(line)
      endwhile
      close, cfg
      free_lun,cfg

      if N_elements(bindir) gt 0 then begin
          bidx = (where(pdirs eq bindir))[0]
          if bidx le -1 then begin
              pdirs= [ bindir, pdirs ]
              bidx = 0
          endif
      endif
      ;;print, "bindir-idx = ",bidx
  endif
  ;;
  ;; make a GUI
  ;;
  base = widget_base(Title="MIA+EWS Installation Wizard", /Row)
  draw = widget_draw(base, XSize=512, YSize=550, Retain=2)
  form = cw_form(base,$
                 [ '0, LABEL, Welcome to the MIA+EWS, Font=-*-lucida-bold-r-*-*-*-240-*, LEFT',$
                   '0, LABEL, Installation Wizard!, Font=-*-lucida-bold-r-*-*-*-240-*',$
                   '0, LABEL, ',$
                   '0, LABEL, This wizard will guide you through, LEFT',$
                   '0, LABEL, the installation of MIA+EWS., LEFT',$
                   '0, LABEL, ',$
                   '0, LABEL, Installation path:, LEFT',$
                   '0, LABEL, '+cwd+', LEFT',$
                   '0, LABEL, ',$
                   '0, LABEL, Make-command:, LEFT',$
                   '0, TEXT, '+make+', WIDTH=42,TAG=make',$
                   '0, LABEL, ',$
                   '0, LABEL, Browser for Help-pages:, LEFT',$
                   '0, TEXT, '+browser+', WIDTH=42,TAG=Browser',$
                   '0, LABEL, ',$
                   '0, LABEL, Directory for executable:, LEFT',$
                   '0, DROPLIST, '+strjoin(pdirs,"|")+',TAG=PATH, SET_VALUE='+string(bidx),$
                   '0, LABEL, ',$
                   '1, Base, , Row',$
                     '0, BUTTON, Cancel,  TAG=cancel',$
                     '2, BUTTON, Install, TAG=go',$
                   '0, LABEL, ' ], /Column )

  Widget_Control, base, /REALIZE
  Widget_Control, draw, GET_VALUE=drawID
  wset, drawID
  tv, iwiz, /true

  xmanager, 'installwiz', base
end
